@@ -5,7 +5,7 @@ Core Flytekit
 
 .. currentmodule:: flytekit
 
-This package contains all of the most common abstractions you'll need to write Flyte workflows, and extend Flytekit.
+This package contains all of the most common abstractions you'll need to write Flyte workflows and extend Flytekit.
 
 Basic Authoring
 ===============
@@ -25,6 +25,8 @@ These are the essentials needed to get started writing tasks and workflows. The
    FlyteContext
    map_task
    ~core.workflow.ImperativeWorkflow
+   ~core.node_creation.create_node
+   FlyteContextManager
 
 Running Locally
 ------------------
@@ -56,7 +58,7 @@ Customizing Tasks & Workflows
    TaskMetadata - Wrapper object that allows users to specify Task
    Resources - Things like CPUs/Memory, etc.
    WorkflowFailurePolicy - Customizes what happens when a workflow fails.
-
+   PodTemplate - Custom PodTemplate for a task.
 
 Dynamic and Nested Workflows
 ==============================
@@ -132,26 +134,48 @@ Secrets and SecurityContext
    Secret
    SecurityContext
 
-"""
 
+Common Flyte IDL Objects
+=========================
+
+.. autosummary::
+   :nosignatures:
+   :template: custom.rst
+   :toctree: generated/
+
+   AuthRole
+   Labels
+   Annotations
+   WorkflowExecutionPhase
+   Blob
+   BlobMetadata
+   Literal
+   Scalar
+   LiteralType
+   BlobType
+"""
 import sys
+from typing import Generator
 
 if sys.version_info < (3, 10):
     from importlib_metadata import entry_points
 else:
     from importlib.metadata import entry_points
 
-import flytekit.plugins  # This will be deprecated, these are the old plugins, the new plugins live in plugins/
 from flytekit.core.base_sql_task import SQLTask
 from flytekit.core.base_task import SecurityContext, TaskMetadata, kwtypes
+from flytekit.core.checkpointer import Checkpoint
 from flytekit.core.condition import conditional
 from flytekit.core.container_task import ContainerTask
 from flytekit.core.context_manager import ExecutionParameters, FlyteContext, FlyteContextManager
 from flytekit.core.data_persistence import DataPersistence, DataPersistencePlugins
 from flytekit.core.dynamic_workflow_task import dynamic
+from flytekit.core.gate import approve, sleep, wait_for_input
+from flytekit.core.hash import HashMethod
 from flytekit.core.launch_plan import LaunchPlan
 from flytekit.core.map_task import map_task
 from flytekit.core.notification import Email, PagerDuty, Slack
+from flytekit.core.pod_template import PodTemplate
 from flytekit.core.python_function_task import PythonFunctionTask, PythonInstanceTask
 from flytekit.core.reference import get_reference_entity
 from flytekit.core.reference_entity import LaunchPlanReference, TaskReference, WorkflowReference
@@ -160,11 +184,25 @@ from flytekit.core.schedule import CronSchedule, FixedRate
 from flytekit.core.task import Secret, reference_task, task
 from flytekit.core.workflow import ImperativeWorkflow as Workflow
 from flytekit.core.workflow import WorkflowFailurePolicy, reference_workflow, workflow
+from flytekit.deck import Deck
+from flytekit.extras import pytorch, sklearn, tensorflow
 from flytekit.extras.persistence import GCSPersistence, HttpPersistence, S3Persistence
 from flytekit.loggers import logger
-from flytekit.types import directory, file, schema
+from flytekit.models.common import Annotations, AuthRole, Labels
+from flytekit.models.core.execution import WorkflowExecutionPhase
+from flytekit.models.core.types import BlobType
+from flytekit.models.documentation import Description, Documentation, SourceCode
+from flytekit.models.literals import Blob, BlobMetadata, Literal, Scalar
+from flytekit.models.types import LiteralType
+from flytekit.types import directory, file, numpy, schema
+from flytekit.types.structured.structured_dataset import (
+    StructuredDataset,
+    StructuredDatasetFormat,
+    StructuredDatasetTransformerEngine,
+    StructuredDatasetType,
+)
 
-__version__ = "0.22.1"
+__version__ = "1.3.3"
 
 
 def current_context() -> ExecutionParameters:
@@ -183,6 +221,10 @@ def current_context() -> ExecutionParameters:
     return FlyteContextManager.current_context().execution_state.user_space_params
 
 
+def new_context() -> Generator[FlyteContext, None, None]:
+    return FlyteContextManager.with_context(FlyteContextManager.current_context().new_builder())
+
+
 def load_implicit_plugins():
     """
     This method allows loading all plugins that have the entrypoint specification. This uses the plugin loading
