@@ -1,3 +1,4 @@
+import datetime
 import typing
 
 from flyteidl.admin import common_pb2 as _common_pb2
@@ -11,6 +12,8 @@ from flyteidl.admin import task_execution_pb2 as _task_execution_pb2
 from flyteidl.admin import task_pb2 as _task_pb2
 from flyteidl.admin import workflow_attributes_pb2 as _workflow_attributes_pb2
 from flyteidl.admin import workflow_pb2 as _workflow_pb2
+from flyteidl.service import dataproxy_pb2 as _data_proxy_pb2
+from google.protobuf.duration_pb2 import Duration
 
 from flytekit.clients.raw import RawSynchronousFlyteClient as _RawSynchronousFlyteClient
 from flytekit.models import common as _common
@@ -560,7 +563,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
     def recover_execution(self, id, name: str = None):
         """
         Recreates a previously-run workflow execution that will only start executing from the last known failure point.
-        :param flytekit.common.core.identifier.WorkflowExecutionIdentifier id:
+        :param flytekit.models.core.identifier.WorkflowExecutionIdentifier id:
         :param name str: Optional name to assign to the newly created execution.
         :rtype: flytekit.models.core.identifier.WorkflowExecutionIdentifier
         """
@@ -572,7 +575,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
 
     def get_execution(self, id):
         """
-        :param flytekit.common.core.identifier.WorkflowExecutionIdentifier id:
+        :param flytekit.models.core.identifier.WorkflowExecutionIdentifier id:
         :rtype: flytekit.models.execution.Execution
         """
         return _execution.Execution.from_flyte_idl(
@@ -638,7 +641,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
 
     def terminate_execution(self, id, cause):
         """
-        :param flytekit.common.core.identifier.WorkflowExecutionIdentifier id:
+        :param flytekit.models.core.identifier.WorkflowExecutionIdentifier id:
         :param Text cause:
         """
         super(SynchronousFlyteClient, self).terminate_execution(
@@ -647,7 +650,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
 
     def relaunch_execution(self, id, name=None):
         """
-        :param flytekit.common.core.identifier.WorkflowExecutionIdentifier id:
+        :param flytekit.models.core.identifier.WorkflowExecutionIdentifier id:
         :param Text name: [Optional] name for the new execution. If not specified, a randomly generated name will be
             used
         :returns: The unique identifier for the new execution.
@@ -676,12 +679,11 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             )
         )
 
-    def get_node_execution_data(self, node_execution_identifier):
+    def get_node_execution_data(self, node_execution_identifier) -> _execution.NodeExecutionGetDataResponse:
         """
         Returns signed URLs to LiteralMap blobs for a node execution's inputs and outputs (when available).
 
         :param flytekit.models.core.identifier.NodeExecutionIdentifier node_execution_identifier:
-        :rtype: flytekit.models.execution.NodeExecutionGetDataResponse
         """
         return _execution.NodeExecutionGetDataResponse.from_flyte_idl(
             super(SynchronousFlyteClient, self).get_node_execution_data(
@@ -974,3 +976,45 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
                 resource_type=resource_type,
             )
         )
+
+    def get_upload_signed_url(
+        self, project: str, domain: str, content_md5: bytes, filename: str = None, expires_in: datetime.timedelta = None
+    ) -> _data_proxy_pb2.CreateUploadLocationResponse:
+        """
+        Get a signed url to be used during fast registration
+        :param str project: Project to create the upload location for
+        :param str domain: Domain to create the upload location for
+        :param bytes content_md5: ContentMD5 restricts the upload location to the specific MD5 provided. The content_md5
+            will also appear in the generated path.
+        :param str filename: [Optional] If provided this specifies a desired suffix for the generated location
+        :param datetime.timedelta expires_in: [Optional] If provided this defines a requested expiration duration for
+            the generated url
+        :rtype: flyteidl.service.dataproxy_pb2.CreateUploadLocationResponse
+        """
+        expires_in_pb = None
+        if expires_in:
+            expires_in_pb = Duration()
+            expires_in_pb.FromTimedelta(expires_in)
+        return super(SynchronousFlyteClient, self).create_upload_location(
+            _data_proxy_pb2.CreateUploadLocationRequest(
+                project=project,
+                domain=domain,
+                content_md5=content_md5,
+                filename=filename,
+                expires_in=expires_in_pb,
+            )
+        )
+
+    def get_download_signed_url(
+        self, native_url: str, expires_in: datetime.timedelta = None
+    ) -> _data_proxy_pb2.CreateUploadLocationResponse:
+        expires_in_pb = None
+        if expires_in:
+            expires_in_pb = Duration()
+            expires_in_pb.FromTimedelta(expires_in)
+        return super(SynchronousFlyteClient, self).create_download_location(
+            _data_proxy_pb2.CreateDownloadLocationRequest(
+                native_url=native_url,
+                expires_in=expires_in_pb,
+            )
+        )
