@@ -1,117 +1,32 @@
+from __future__ import annotations
+
+import base64 as _base64
+import ssl
 import subprocess
 import time
-from typing import List
+import typing
+from typing import Optional
 
-import six as _six
+import grpc
+import requests as _requests
+from flyteidl.admin.project_pb2 import ProjectListRequest
+from flyteidl.admin.signal_pb2 import SignalList, SignalListRequest, SignalSetRequest, SignalSetResponse
 from flyteidl.service import admin_pb2_grpc as _admin_service
+from flyteidl.service import auth_pb2
+from flyteidl.service import auth_pb2_grpc as auth_service
+from flyteidl.service import dataproxy_pb2 as _dataproxy_pb2
+from flyteidl.service import dataproxy_pb2_grpc as dataproxy_service
+from flyteidl.service import signal_pb2_grpc as signal_service
+from flyteidl.service.dataproxy_pb2_grpc import DataProxyServiceStub
 from google.protobuf.json_format import MessageToJson as _MessageToJson
-from grpc import RpcError as _RpcError
-from grpc import StatusCode as _GrpcStatusCode
-from grpc import insecure_channel as _insecure_channel
-from grpc import secure_channel as _secure_channel
-from grpc import ssl_channel_credentials as _ssl_channel_credentials
 
 from flytekit.clis.auth import credentials as _credentials_access
-from flytekit.clis.sdk_in_container import basic_auth as _basic_auth
-from flytekit.common.exceptions import user as _user_exceptions
-from flytekit.configuration import creds as _creds_config
-from flytekit.configuration.creds import _DEPRECATED_CLIENT_CREDENTIALS_SCOPE as _DEPRECATED_SCOPE
-from flytekit.configuration.creds import CLIENT_ID as _CLIENT_ID
-from flytekit.configuration.creds import COMMAND as _COMMAND
-from flytekit.configuration.creds import OAUTH_SCOPES
-from flytekit.configuration.platform import AUTH as _AUTH
+from flytekit.configuration import AuthType, PlatformConfig
+from flytekit.exceptions import user as _user_exceptions
+from flytekit.exceptions.user import FlyteAuthenticationException
 from flytekit.loggers import cli_logger
 
-
-def _refresh_credentials_standard(flyte_client):
-    """
-    This function is used when the configuration value for AUTH_MODE is set to 'standard'.
-    This either fetches the existing access token or initiates the flow to request a valid access token and store it.
-    :param flyte_client: RawSynchronousFlyteClient
-    :return:
-    """
-
-    client = _credentials_access.get_client(flyte_client.url)
-    if client.can_refresh_token:
-        client.refresh_access_token()
-
-    flyte_client.set_access_token(client.credentials.access_token)
-
-
-def _get_basic_flow_scopes() -> List[str]:
-    """
-    Merge the scope value between the old scope config option and the new list option.
-
-    :return: The scopes to use for basic auth flow.
-    """
-    deprecated_single_scope = _DEPRECATED_SCOPE.get()
-    if deprecated_single_scope:
-        return [deprecated_single_scope]
-    oauth_scopes = OAUTH_SCOPES.get()
-    if "openid" in oauth_scopes:
-        cli_logger.warning("Basic flow authentication should never use openid.")
-
-    return oauth_scopes
-
-
-def _refresh_credentials_basic(flyte_client):
-    """
-    This function is used by the _handle_rpc_error() decorator, depending on the AUTH_MODE config object. This handler
-    is meant for SDK use-cases of auth (like pyflyte, or when users call SDK functions that require access to Admin,
-    like when waiting for another workflow to complete from within a task). This function uses basic auth, which means
-    the credentials for basic auth must be present from wherever this code is running.
-
-    :param flyte_client: RawSynchronousFlyteClient
-    :return:
-    """
-    auth_endpoints = _credentials_access.get_authorization_endpoints(flyte_client.url)
-    token_endpoint = auth_endpoints.token_endpoint
-    client_secret = _basic_auth.get_secret()
-    cli_logger.debug(
-        "Basic authorization flow with client id {} scope {}".format(_CLIENT_ID.get(), _get_basic_flow_scopes())
-    )
-    authorization_header = _basic_auth.get_basic_authorization_header(_CLIENT_ID.get(), client_secret)
-    token, expires_in = _basic_auth.get_token(token_endpoint, authorization_header, _get_basic_flow_scopes())
-    cli_logger.info("Retrieved new token, expires in {}".format(expires_in))
-    flyte_client.set_access_token(token)
-
-
-def _refresh_credentials_from_command(flyte_client):
-    """
-    This function is used when the configuration value for AUTH_MODE is set to 'external_process'.
-    It reads an id token generated by an external process started by running the 'command'.
-
-    :param flyte_client: RawSynchronousFlyteClient
-    :return:
-    """
-
-    command = _COMMAND.get()
-    cli_logger.debug("Starting external process to generate id token. Command {}".format(command))
-    try:
-        output = subprocess.run(command, capture_output=True, text=True, check=True)
-    except subprocess.CalledProcessError as e:
-        cli_logger.error("Failed to generate token from command {}".format(command))
-        raise _user_exceptions.FlyteAuthenticationException(
-            "Problems refreshing token with command: " + _six.text_type(e)
-        )
-    flyte_client.set_access_token(output.stdout.strip())
-
-
-def _refresh_credentials_noop(flyte_client):
-    pass
-
-
-def _get_refresh_handler(auth_mode):
-    if auth_mode == "standard":
-        return _refresh_credentials_standard
-    elif auth_mode == "basic":
-        return _refresh_credentials_basic
-    elif auth_mode == "external_process":
-        return _refresh_credentials_from_command
-    else:
-        raise ValueError(
-            "Invalid auth mode [{}] specified. Please update the creds config to use a valid value".format(auth_mode)
-        )
+_utf_8 = "utf-8"
 
 
 def _handle_rpc_error(retry=False):
@@ -119,38 +34,37 @@ def _handle_rpc_error(retry=False):
         def handler(*args, **kwargs):
             """
             Wraps rpc errors as Flyte exceptions and handles authentication the client.
-            :param args:
-            :param kwargs:
-            :return:
             """
             max_retries = 3
             max_wait_time = 1000
-            try:
-                for i in range(max_retries):
-                    try:
-                        return fn(*args, **kwargs)
-                    except _RpcError as e:
-                        if e.code() == _GrpcStatusCode.UNAUTHENTICATED:
-                            # Always retry auth errors.
-                            if i == (max_retries - 1):
-                                # Exit the loop and wrap the authentication error.
-                                raise _user_exceptions.FlyteAuthenticationException(_six.text_type(e))
-                            refresh_handler_fn = _get_refresh_handler(_creds_config.AUTH_MODE.get())
-                            refresh_handler_fn(args[0])
+
+            for i in range(max_retries):
+                try:
+                    return fn(*args, **kwargs)
+                except grpc.RpcError as e:
+                    if e.code() == grpc.StatusCode.UNAUTHENTICATED:
+                        # Always retry auth errors.
+                        if i == (max_retries - 1):
+                            # Exit the loop and wrap the authentication error.
+                            raise _user_exceptions.FlyteAuthenticationException(str(e))
+                        cli_logger.debug(f"Unauthenticated RPC error {e}, refreshing credentials and retrying\n")
+                        args[0].refresh_credentials()
+                    elif e.code() == grpc.StatusCode.ALREADY_EXISTS:
+                        # There are two cases that we should throw error immediately
+                        # 1. Entity already exists when we register entity
+                        # 2. Entity not found when we fetch entity
+                        raise _user_exceptions.FlyteEntityAlreadyExistsException(e)
+                    elif e.code() == grpc.StatusCode.NOT_FOUND:
+                        raise _user_exceptions.FlyteEntityNotExistException(e)
+                    else:
+                        # No more retries if retry=False or max_retries reached.
+                        if (retry is False) or i == (max_retries - 1):
+                            raise
                         else:
-                            # No more retries if retry=False or max_retries reached.
-                            if (retry is False) or i == (max_retries - 1):
-                                raise
-                            else:
-                                # Retry: Start with 200ms wait-time and exponentially back-off up to 1 second.
-                                wait_time = min(200 * (2 ** i), max_wait_time)
-                                cli_logger.error(f"Non-auth RPC error {e}, sleeping {wait_time}ms and retrying")
-                                time.sleep(wait_time / 1000)
-            except _RpcError as e:
-                if e.code() == _GrpcStatusCode.ALREADY_EXISTS:
-                    raise _user_exceptions.FlyteEntityAlreadyExistsException(_six.text_type(e))
-                else:
-                    raise
+                            # Retry: Start with 200ms wait-time and exponentially back-off up to 1 second.
+                            wait_time = min(200 * (2**i), max_wait_time)
+                            cli_logger.error(f"Non-auth RPC error {e}, sleeping {wait_time}ms and retrying")
+                            time.sleep(wait_time / 1000)
 
         return handler
 
@@ -161,12 +75,13 @@ def _handle_invalid_create_request(fn):
     def handler(self, create_request):
         try:
             fn(self, create_request)
-        except _RpcError as e:
-            if e.code() == _GrpcStatusCode.INVALID_ARGUMENT:
+        except grpc.RpcError as e:
+            if e.code() == grpc.StatusCode.INVALID_ARGUMENT:
                 cli_logger.error("Error creating Flyte entity because of invalid arguments. Create request: ")
                 cli_logger.error(_MessageToJson(create_request))
-
-            # In any case, re-raise since we're not truly handling the error here
+                cli_logger.error("Details returned from the flyte admin: ")
+                cli_logger.error(e.details)
+            # Re-raise since we're not  handling the error here and add the create_request details
             raise e
 
     return handler
@@ -177,54 +92,235 @@ class RawSynchronousFlyteClient(object):
     This is a thin synchronous wrapper around the auto-generated GRPC stubs for communicating with the admin service.
 
     This client should be usable regardless of environment in which this is used. In other words, configurations should
-    be explicit as opposed to inferred from the environment or a configuration file.
+    be explicit as opposed to inferred from the environment or a configuration file. To create a client,
+
+    .. code-block:: python
+
+        from flytekit.configuration import PlatformConfig
+        RawSynchronousFlyteClient(PlatformConfig(endpoint="a.b.com", insecure=True))  # or
+        SynchronousFlyteClient(PlatformConfig(endpoint="a.b.com", insecure=True))
     """
 
-    def __init__(self, url, insecure=False, credentials=None, options=None):
+    _dataproxy_stub: DataProxyServiceStub
+
+    def __init__(self, cfg: PlatformConfig, **kwargs):
         """
         Initializes a gRPC channel to the given Flyte Admin service.
 
-        :param Text url: The URL (including port if necessary) to connect to the appropriate Flyte Admin Service.
-        :param bool insecure: [Optional] Whether to use an insecure connection, default False
-        :param Text credentials: [Optional] If provided, a secure channel will be opened with the Flyte Admin Service.
-        :param dict[Text, Text] options: [Optional] A dict of key-value string pairs for configuring the gRPC core
-            runtime.
-        :param list[(Text,Text)] metadata: [Optional] metadata pairs to be transmitted to the
-            service-side of the RPC.
+        Args:
+          url: The server address.
+          insecure: if insecure is desired
         """
-        self._channel = None
-        self._url = url
-
-        if insecure:
-            self._channel = _insecure_channel(url, options=list((options or {}).items()))
+        self._cfg = cfg
+        if cfg.insecure:
+            self._channel = grpc.insecure_channel(cfg.endpoint, **kwargs)
+        elif cfg.insecure_skip_verify:
+            # Get port from endpoint or use 443
+            endpoint_parts = cfg.endpoint.rsplit(":", 1)
+            if len(endpoint_parts) == 2 and endpoint_parts[1].isdigit():
+                server_address = tuple(endpoint_parts)
+            else:
+                server_address = (cfg.endpoint, "443")
+            cert = ssl.get_server_certificate(server_address)
+            credentials = grpc.ssl_channel_credentials(str.encode(cert))
+            options = kwargs.get("options", [])
+            self._channel = grpc.secure_channel(
+                target=cfg.endpoint,
+                credentials=credentials,
+                options=options,
+                compression=kwargs.get("compression", None),
+            )
         else:
-            self._channel = _secure_channel(
-                url,
-                credentials or _ssl_channel_credentials(),
-                options=list((options or {}).items()),
+            if "credentials" not in kwargs:
+                credentials = grpc.ssl_channel_credentials(
+                    root_certificates=kwargs.get("root_certificates", None),
+                    private_key=kwargs.get("private_key", None),
+                    certificate_chain=kwargs.get("certificate_chain", None),
+                )
+            else:
+                credentials = kwargs["credentials"]
+            self._channel = grpc.secure_channel(
+                target=cfg.endpoint,
+                credentials=credentials,
+                options=kwargs.get("options", None),
+                compression=kwargs.get("compression", None),
             )
         self._stub = _admin_service.AdminServiceStub(self._channel)
+        self._auth_stub = auth_service.AuthMetadataServiceStub(self._channel)
+        self._signal = signal_service.SignalServiceStub(self._channel)
+        try:
+            resp = self._auth_stub.GetPublicClientConfig(auth_pb2.PublicClientAuthConfigRequest())
+            self._public_client_config = resp
+        except grpc.RpcError:
+            cli_logger.debug("No public client auth config found, skipping.")
+            self._public_client_config = None
+        try:
+            resp = self._auth_stub.GetOAuth2Metadata(auth_pb2.OAuth2MetadataRequest())
+            self._oauth2_metadata = resp
+        except grpc.RpcError:
+            cli_logger.debug("No OAuth2 Metadata found, skipping.")
+            self._oauth2_metadata = None
+        self._dataproxy_stub = dataproxy_service.DataProxyServiceStub(self._channel)
+
+        cli_logger.info(
+            f"Flyte Client configured -> {self._cfg.endpoint} in {'insecure' if self._cfg.insecure else 'secure'} mode."
+        )
+        # metadata will hold the value of the token to send to the various endpoints.
         self._metadata = None
-        if _AUTH.get():
-            self.force_auth_flow()
+
+    @classmethod
+    def with_root_certificate(cls, cfg: PlatformConfig, root_cert_file: str) -> RawSynchronousFlyteClient:
+        b = None
+        with open(root_cert_file, "rb") as fp:
+            b = fp.read()
+        return RawSynchronousFlyteClient(cfg, credentials=grpc.ssl_channel_credentials(root_certificates=b))
+
+    @property
+    def public_client_config(self) -> Optional[auth_pb2.PublicClientAuthConfigResponse]:
+        return self._public_client_config
+
+    @property
+    def oauth2_metadata(self) -> Optional[auth_pb2.OAuth2MetadataResponse]:
+        return self._oauth2_metadata
 
     @property
     def url(self) -> str:
-        return self._url
+        return self._cfg.endpoint
 
-    def set_access_token(self, access_token):
+    def _refresh_credentials_standard(self):
+        """
+        This function is used when the configuration value for AUTH_MODE is set to 'standard'.
+        This either fetches the existing access token or initiates the flow to request a valid access token and store it.
+        :param self: RawSynchronousFlyteClient
+        :return:
+        """
+        authorization_header_key = self.public_client_config.authorization_metadata_key or None
+        if not self.oauth2_metadata or not self.public_client_config:
+            raise ValueError(
+                "Raw Flyte client attempting client credentials flow but no response from Admin detected. "
+                "Check your Admin server's .well-known endpoints to make sure they're working as expected."
+            )
+
+        client = _credentials_access.get_client(
+            redirect_endpoint=self.public_client_config.redirect_uri,
+            client_id=self.public_client_config.client_id,
+            scopes=self.public_client_config.scopes,
+            auth_endpoint=self.oauth2_metadata.authorization_endpoint,
+            token_endpoint=self.oauth2_metadata.token_endpoint,
+        )
+
+        if client.has_valid_credentials and not self.check_access_token(client.credentials.access_token):
+            # When Python starts up, if credentials have been stored in the keyring, then the AuthorizationClient
+            # will have read them into its _credentials field, but it won't be in the RawSynchronousFlyteClient's
+            # metadata field yet. Therefore, if there's a mismatch, copy it over.
+            self.set_access_token(client.credentials.access_token, authorization_header_key)
+            return
+
+        try:
+            client.refresh_access_token()
+        except ValueError:
+            client.start_authorization_flow()
+
+        self.set_access_token(client.credentials.access_token, authorization_header_key)
+
+    def _refresh_credentials_basic(self):
+        """
+        This function is used by the _handle_rpc_error() decorator, depending on the AUTH_MODE config object. This handler
+        is meant for SDK use-cases of auth (like pyflyte, or when users call SDK functions that require access to Admin,
+        like when waiting for another workflow to complete from within a task). This function uses basic auth, which means
+        the credentials for basic auth must be present from wherever this code is running.
+
+        :param self: RawSynchronousFlyteClient
+        :return:
+        """
+        if not self.oauth2_metadata or not self.public_client_config:
+            raise ValueError(
+                "Raw Flyte client attempting client credentials flow but no response from Admin detected. "
+                "Check your Admin server's .well-known endpoints to make sure they're working as expected."
+            )
+
+        token_endpoint = self.oauth2_metadata.token_endpoint
+        scopes = self._cfg.scopes or self.public_client_config.scopes
+        scopes = ",".join(scopes)
+
+        # Note that unlike the Pkce flow, the client ID does not come from Admin.
+        client_secret = self._cfg.client_credentials_secret
+        if not client_secret:
+            raise FlyteAuthenticationException("No client credentials secret provided in the config")
+        cli_logger.debug(f"Basic authorization flow with client id {self._cfg.client_id} scope {scopes}")
+        authorization_header = get_basic_authorization_header(self._cfg.client_id, client_secret)
+        token, expires_in = get_token(token_endpoint, authorization_header, scopes)
+        cli_logger.info("Retrieved new token, expires in {}".format(expires_in))
+        authorization_header_key = self.public_client_config.authorization_metadata_key or None
+        self.set_access_token(token, authorization_header_key)
+
+    def _refresh_credentials_from_command(self):
+        """
+        This function is used when the configuration value for AUTH_MODE is set to 'external_process'.
+        It reads an id token generated by an external process started by running the 'command'.
+
+        :param self: RawSynchronousFlyteClient
+        :return:
+        """
+        command = self._cfg.command
+        if not command:
+            raise FlyteAuthenticationException("No command specified in configuration for command authentication")
+        cli_logger.debug("Starting external process to generate id token. Command {}".format(command))
+        try:
+            output = subprocess.run(command, capture_output=True, text=True, check=True)
+        except subprocess.CalledProcessError as e:
+            cli_logger.error("Failed to generate token from command {}".format(command))
+            raise _user_exceptions.FlyteAuthenticationException("Problems refreshing token with command: " + str(e))
+        authorization_header_key = self.public_client_config.authorization_metadata_key or None
+        if not authorization_header_key:
+            self.set_access_token(output.stdout.strip())
+        self.set_access_token(output.stdout.strip(), authorization_header_key)
+
+    def _refresh_credentials_noop(self):
+        pass
+
+    def refresh_credentials(self):
+        cfg_auth = self._cfg.auth_mode
+        if type(cfg_auth) is str:
+            try:
+                cfg_auth = AuthType[cfg_auth.upper()]
+            except KeyError:
+                cli_logger.warning(f"Authentication type {cfg_auth} does not exist, defaulting to standard")
+                cfg_auth = AuthType.STANDARD
+
+        if cfg_auth == AuthType.STANDARD or cfg_auth == AuthType.PKCE:
+            return self._refresh_credentials_standard()
+        elif cfg_auth == AuthType.BASIC or cfg_auth == AuthType.CLIENT_CREDENTIALS or cfg_auth == AuthType.CLIENTSECRET:
+            return self._refresh_credentials_basic()
+        elif cfg_auth == AuthType.EXTERNAL_PROCESS or cfg_auth == AuthType.EXTERNALCOMMAND:
+            return self._refresh_credentials_from_command()
+        else:
+            raise ValueError(
+                f"Invalid auth mode [{cfg_auth}] specified." f"Please update the creds config to use a valid value"
+            )
+
+    def set_access_token(self, access_token: str, authorization_header_key: Optional[str] = "authorization"):
         # Always set the header to lower-case regardless of what the config is. The grpc libraries that Admin uses
         # to parse the metadata don't change the metadata, but they do automatically lower the key you're looking for.
+        cli_logger.debug(f"Adding authorization header. Header name: {authorization_header_key}.")
         self._metadata = [
             (
-                _creds_config.AUTHORIZATION_METADATA_KEY.get().lower(),
-                "Bearer {}".format(access_token),
+                authorization_header_key,
+                f"Bearer {access_token}",
             )
         ]
 
-    def force_auth_flow(self):
-        refresh_handler_fn = _get_refresh_handler(_creds_config.AUTH_MODE.get())
-        refresh_handler_fn(self)
+    def check_access_token(self, access_token: str) -> bool:
+        """
+        This checks to see if the given access token is the same as the one already stored in the client. The reason
+        this is useful is so that we can prevent unnecessary refreshing of tokens.
+
+        :param access_token: The access token to check
+        :return: If no access token is stored, or if the stored token doesn't match, return False.
+        """
+        if self._metadata is None:
+            return False
+        return access_token == self._metadata[0][1].replace("Bearer ", "")
 
     ####################################################################################################################
     #
@@ -313,6 +409,20 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.GetTask(get_object_request, metadata=self._metadata)
 
+    @_handle_rpc_error(retry=True)
+    def set_signal(self, signal_set_request: SignalSetRequest) -> SignalSetResponse:
+        """
+        This sets a signal
+        """
+        return self._signal.SetSignal(signal_set_request, metadata=self._metadata)
+
+    @_handle_rpc_error(retry=True)
+    def list_signals(self, signal_list_request: SignalListRequest) -> SignalList:
+        """
+        This lists signals
+        """
+        return self._signal.ListSignals(signal_list_request, metadata=self._metadata)
+
     ####################################################################################################################
     #
     #  Workflow Endpoints
@@ -655,12 +765,14 @@ class RawSynchronousFlyteClient(object):
     ####################################################################################################################
 
     @_handle_rpc_error(retry=True)
-    def list_projects(self, project_list_request):
+    def list_projects(self, project_list_request: typing.Optional[ProjectListRequest] = None):
         """
         This will return a list of the projects registered with the Flyte Admin Service
         :param flyteidl.admin.project_pb2.ProjectListRequest project_list_request:
         :rtype: flyteidl.admin.project_pb2.Projects
         """
+        if project_list_request is None:
+            project_list_request = ProjectListRequest()
         return self._stub.ListProjects(project_list_request, metadata=self._metadata)
 
     @_handle_rpc_error()
@@ -741,3 +853,70 @@ class RawSynchronousFlyteClient(object):
 
     # TODO: (P2) Implement the event endpoints in case there becomes a use-case for third-parties to submit events
     # through the client in Python.
+
+    ####################################################################################################################
+    #
+    #  Data proxy endpoints
+    #
+    ####################################################################################################################
+    @_handle_rpc_error(retry=True)
+    def create_upload_location(
+        self, create_upload_location_request: _dataproxy_pb2.CreateUploadLocationRequest
+    ) -> _dataproxy_pb2.CreateUploadLocationResponse:
+        """
+        Get a signed url to be used during fast registration
+        :param flyteidl.service.dataproxy_pb2.CreateUploadLocationRequest create_upload_location_request:
+        :rtype: flyteidl.service.dataproxy_pb2.CreateUploadLocationResponse
+        """
+        return self._dataproxy_stub.CreateUploadLocation(create_upload_location_request, metadata=self._metadata)
+
+    @_handle_rpc_error(retry=True)
+    def create_download_location(
+        self, create_download_location_request: _dataproxy_pb2.CreateDownloadLocationRequest
+    ) -> _dataproxy_pb2.CreateDownloadLocationResponse:
+        """
+        Get a signed url to be used during fast registration
+        :param flyteidl.service.dataproxy_pb2.CreateDownloadLocationRequest create_download_location_request:
+        :rtype: flyteidl.service.dataproxy_pb2.CreateDownloadLocationResponse
+        """
+        return self._dataproxy_stub.CreateDownloadLocation(create_download_location_request, metadata=self._metadata)
+
+
+def get_token(token_endpoint, authorization_header, scope):
+    """
+    :param Text token_endpoint:
+    :param Text authorization_header: This is the value for the "Authorization" key. (eg 'Bearer abc123')
+    :param Text scope:
+    :rtype: (Text,Int) The first element is the access token retrieved from the IDP, the second is the expiration
+            in seconds
+    """
+    headers = {
+        "Authorization": authorization_header,
+        "Cache-Control": "no-cache",
+        "Accept": "application/json",
+        "Content-Type": "application/x-www-form-urlencoded",
+    }
+    body = {
+        "grant_type": "client_credentials",
+    }
+    if scope is not None:
+        body["scope"] = scope
+    response = _requests.post(token_endpoint, data=body, headers=headers)
+    if response.status_code != 200:
+        cli_logger.error("Non-200 ({}) received from IDP: {}".format(response.status_code, response.text))
+        raise FlyteAuthenticationException("Non-200 received from IDP")
+
+    response = response.json()
+    return response["access_token"], response["expires_in"]
+
+
+def get_basic_authorization_header(client_id, client_secret):
+    """
+    This function transforms the client id and the client secret into a header that conforms with http basic auth.
+    It joins the id and the secret with a : then base64 encodes it, then adds the appropriate text.
+    :param Text client_id:
+    :param Text client_secret:
+    :rtype: Text
+    """
+    concated = "{}:{}".format(client_id, client_secret)
+    return "Basic {}".format(_base64.b64encode(concated.encode(_utf_8)).decode(_utf_8))
