@@ -21,10 +21,10 @@ import collections
 import datetime
 from abc import abstractmethod
 from dataclasses import dataclass
-from typing import Any, Dict, Generic, List, Optional, Tuple, Type, TypeVar, Union
+from typing import Any, Dict, Generic, List, Optional, OrderedDict, Tuple, Type, TypeVar, Union
 
-from flytekit.common.tasks.sdk_runnable import ExecutionParameters
-from flytekit.core.context_manager import FlyteContext, FlyteContextManager, FlyteEntities, SerializationSettings
+from flytekit.configuration import SerializationSettings
+from flytekit.core.context_manager import ExecutionParameters, FlyteContext, FlyteContextManager, FlyteEntities
 from flytekit.core.interface import Interface, transform_interface_to_typed_interface
 from flytekit.core.local_cache import LocalTaskCache
 from flytekit.core.promise import (
@@ -32,22 +32,25 @@ from flytekit.core.promise import (
     VoidPromise,
     create_and_link_node,
     create_task_output,
+    extract_obj_name,
     flyte_entity_call_handler,
     translate_inputs_to_literals,
 )
 from flytekit.core.tracker import TrackedInstance
 from flytekit.core.type_engine import TypeEngine
+from flytekit.deck.deck import Deck
 from flytekit.loggers import logger
 from flytekit.models import dynamic_job as _dynamic_job
 from flytekit.models import interface as _interface_models
 from flytekit.models import literals as _literal_models
 from flytekit.models import task as _task_model
 from flytekit.models.core import workflow as _workflow_model
+from flytekit.models.documentation import Description, Documentation
 from flytekit.models.interface import Variable
 from flytekit.models.security import SecurityContext
 
 
-def kwtypes(**kwargs) -> Dict[str, Type]:
+def kwtypes(**kwargs) -> OrderedDict[str, Type]:
     """
     This is a small helper function to convert the keyword arguments to an OrderedDict of types.
 
@@ -71,6 +74,7 @@ class TaskMetadata(object):
 
     Args:
         cache (bool): Indicates if caching should be enabled. See :std:ref:`Caching <cookbook:caching>`
+        cache_serialize (bool): Indicates if identical (ie. same inputs) instances of this task should be executed in serial when caching is enabled. See :std:ref:`Caching <cookbook:caching>`
         cache_version (str): Version to be used for the cached value
         interruptible (Optional[bool]): Indicates that this task can be interrupted and/or scheduled on nodes with
             lower QoS guarantees that can include pre-emption. This can reduce the monetary cost executions incur at the
@@ -81,14 +85,17 @@ class TaskMetadata(object):
         timeout (Optional[Union[datetime.timedelta, int]]): the max amount of time for which one execution of this task
             should be executed for. The execution will be terminated if the runtime exceeds the given timeout
             (approximately)
+        pod_template_name (Optional[str]): the name of existing PodTemplate resource in the cluster which will be used in this task.
     """
 
     cache: bool = False
+    cache_serialize: bool = False
     cache_version: str = ""
     interruptible: Optional[bool] = None
     deprecated: str = ""
     retries: int = 0
     timeout: Optional[Union[datetime.timedelta, int]] = None
+    pod_template_name: Optional[str] = None
 
     def __post_init__(self):
         if self.timeout:
@@ -98,6 +105,8 @@ class TaskMetadata(object):
                 raise ValueError("timeout should be duration represented as either a datetime.timedelta or int seconds")
         if self.cache and not self.cache_version:
             raise ValueError("Caching is enabled ``cache=True`` but ``cache_version`` is not set.")
+        if self.cache_serialize and not self.cache:
+            raise ValueError("Cache serialize is enabled ``cache_serialize=True`` but ``cache`` is not enabled.")
 
     @property
     def retry_strategy(self) -> _literal_models.RetryStrategy:
@@ -119,6 +128,8 @@ class TaskMetadata(object):
             interruptible=self.interruptible,
             discovery_version=self.cache_version,
             deprecated_error_message=self.deprecated,
+            cache_serializable=self.cache_serialize,
+            pod_template_name=self.pod_template_name,
         )
 
 
@@ -149,6 +160,7 @@ class Task(object):
         metadata: Optional[TaskMetadata] = None,
         task_type_version=0,
         security_ctx: Optional[SecurityContext] = None,
+        docs: Optional[Documentation] = None,
         **kwargs,
     ):
         self._task_type = task_type
@@ -157,6 +169,7 @@ class Task(object):
         self._metadata = metadata if metadata else TaskMetadata()
         self._task_type_version = task_type_version
         self._security_ctx = security_ctx
+        self._docs = docs
 
         FlyteEntities.entities.append(self)
 
@@ -188,6 +201,10 @@ class Task(object):
     def security_context(self) -> SecurityContext:
         return self._security_ctx
 
+    @property
+    def docs(self) -> Documentation:
+        return self._docs
+
     def get_type_for_input_var(self, k: str, v: Any) -> type:
         """
         Returns the python native type for the given input variable
@@ -202,7 +219,7 @@ class Task(object):
         """
         return type(v)
 
-    def get_input_types(self) -> Dict[str, type]:
+    def get_input_types(self) -> Optional[Dict[str, type]]:
         """
         Returns python native types for inputs. In case this is not a python native task (base class) and hence
         returns a None. we could deduce the type from literal types, but that is not a required exercise
@@ -210,7 +227,7 @@ class Task(object):
         """
         return None
 
-    def local_execute(self, ctx: FlyteContext, **kwargs) -> Union[Tuple[Promise], Promise, VoidPromise]:
+    def local_execute(self, ctx: FlyteContext, **kwargs) -> Union[Tuple[Promise], Promise, VoidPromise, None]:
         """
         This function is used only in the local execution path and is responsible for calling dispatch execute.
         Use this function when calling a task with native values (or Promises containing Flyte literals derived from
@@ -222,11 +239,10 @@ class Task(object):
         #  Promises as essentially inputs from previous task executions
         #  native constants are just bound to this specific task (default values for a task input)
         #  Also along with promises and constants, there could be dictionary or list of promises or constants
-
         kwargs = translate_inputs_to_literals(
             ctx,
             incoming_values=kwargs,
-            flyte_interface_types=self.interface.inputs,
+            flyte_interface_types=self.interface.inputs,  # type: ignore
             native_types=self.get_input_types(),
         )
         input_literal_map = _literal_models.LiteralMap(literals=kwargs)
@@ -235,7 +251,8 @@ class Task(object):
         if self.metadata.cache:
             # TODO: how to get a nice `native_inputs` here?
             logger.info(
-                f"Checking cache for task named {self.name}, cache version {self.metadata.cache_version} and inputs: {input_literal_map}"
+                f"Checking cache for task named {self.name}, cache version {self.metadata.cache_version} "
+                f"and inputs: {input_literal_map}"
             )
             outputs_literal_map = LocalTaskCache.get(self.name, self.metadata.cache_version, input_literal_map)
             # The cache returns None iff the key does not exist in the cache
@@ -245,18 +262,22 @@ class Task(object):
                 # TODO: need `native_inputs`
                 LocalTaskCache.set(self.name, self.metadata.cache_version, input_literal_map, outputs_literal_map)
                 logger.info(
-                    f"Cache set for task named {self.name}, cache version {self.metadata.cache_version} and inputs: {input_literal_map}"
+                    f"Cache set for task named {self.name}, cache version {self.metadata.cache_version} "
+                    f"and inputs: {input_literal_map}"
                 )
             else:
                 logger.info("Cache hit")
         else:
+            es = ctx.execution_state
+            b = es.user_space_params.with_task_sandbox()
+            ctx = ctx.current_context().with_execution_state(es.with_params(user_space_params=b.build())).build()
             outputs_literal_map = self.dispatch_execute(ctx, input_literal_map)
         outputs_literals = outputs_literal_map.literals
 
         # TODO maybe this is the part that should be done for local execution, we pass the outputs to some special
         #    location, otherwise we dont really need to right? The higher level execute could just handle literalMap
         # After running, we again have to wrap the outputs, if any, back into Promise objects
-        output_names = list(self.interface.outputs.keys())
+        output_names = list(self.interface.outputs.keys())  # type: ignore
         if len(output_names) != len(outputs_literals):
             # Length check, clean up exception
             raise AssertionError(f"Length difference {len(output_names)} {len(outputs_literals)}")
@@ -274,25 +295,31 @@ class Task(object):
     def compile(self, ctx: FlyteContext, *args, **kwargs):
         raise Exception("not implemented")
 
-    def get_container(self, settings: SerializationSettings) -> _task_model.Container:
+    def get_container(self, settings: SerializationSettings) -> Optional[_task_model.Container]:
         """
         Returns the container definition (if any) that is used to run the task on hosted Flyte.
         """
         return None
 
-    def get_k8s_pod(self, settings: SerializationSettings) -> _task_model.K8sPod:
+    def get_k8s_pod(self, settings: SerializationSettings) -> Optional[_task_model.K8sPod]:
         """
         Returns the kubernetes pod definition (if any) that is used to run the task on hosted Flyte.
         """
         return None
 
-    def get_custom(self, settings: SerializationSettings) -> Dict[str, Any]:
+    def get_sql(self, settings: SerializationSettings) -> Optional[_task_model.Sql]:
+        """
+        Returns the Sql definition (if any) that is used to run the task on hosted Flyte.
+        """
+        return None
+
+    def get_custom(self, settings: SerializationSettings) -> Optional[Dict[str, Any]]:
         """
         Return additional plugin-specific custom data (if any) as a serializable dictionary.
         """
         return None
 
-    def get_config(self, settings: SerializationSettings) -> Dict[str, str]:
+    def get_config(self, settings: SerializationSettings) -> Optional[Dict[str, str]]:
         """
         Returns the task config as a serializable dictionary. This task config consists of metadata about the custom
         defined for this task.
@@ -347,6 +374,7 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
         task_config: T,
         interface: Optional[Interface] = None,
         environment: Optional[Dict[str, str]] = None,
+        disable_deck: bool = True,
         **kwargs,
     ):
         """
@@ -360,6 +388,7 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
                 signature of the task
             environment (Optional[Dict[str, str]]): Any environment variables that should be supplied during the
                 execution of the task. Supplied as a dictionary of key/value pairs
+            disable_deck (bool): If true, this task will not output deck html file
         """
         super().__init__(
             task_type=task_type,
@@ -370,6 +399,18 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
         self._python_interface = interface if interface else Interface()
         self._environment = environment if environment else {}
         self._task_config = task_config
+        self._disable_deck = disable_deck
+        if self._python_interface.docstring:
+            if self.docs is None:
+                self._docs = Documentation(
+                    short_description=self._python_interface.docstring.short_description,
+                    long_description=Description(value=self._python_interface.docstring.long_description),
+                )
+            else:
+                if self._python_interface.docstring.short_description:
+                    self._docs.short_description = self._python_interface.docstring.short_description
+                if self._python_interface.docstring.long_description:
+                    self._docs.long_description = Description(value=self._python_interface.docstring.long_description)
 
     # TODO lets call this interface and the other as flyte_interface?
     @property
@@ -409,7 +450,7 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
         Used when constructing the node that encapsulates this task as part of a broader workflow definition.
         """
         return _workflow_model.NodeMetadata(
-            name=f"{self.__module__}.{self.name}",
+            name=extract_obj_name(self.name),
             timeout=self.metadata.timeout,
             retries=self.metadata.retry_strategy,
             interruptible=self.metadata.interruptible,
@@ -423,7 +464,7 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
 
     @property
     def _outputs_interface(self) -> Dict[Any, Variable]:
-        return self.interface.outputs
+        return self.interface.outputs  # type: ignore
 
     def dispatch_execute(
         self, ctx: FlyteContext, input_literal_map: _literal_models.LiteralMap
@@ -440,10 +481,12 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
 
         # Invoked before the task is executed
         new_user_params = self.pre_execute(ctx.user_space_params)
+        from flytekit.deck.deck import _output_deck
 
         # Create another execution context with the new user params, but let's keep the same working dir
         with FlyteContextManager.with_context(
             ctx.with_execution_state(ctx.execution_state.with_params(user_space_params=new_user_params))
+            # type: ignore
         ) as exec_ctx:
             # TODO We could support default values here too - but not part of the plan right now
             # Translate the input literals to Python native
@@ -458,13 +501,14 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
                 logger.exception(f"Exception when executing {e}")
                 raise e
 
-            logger.info(f"Task executed successfully in user level, outputs: {native_outputs}")
+            logger.debug("Task executed successfully in user level")
             # Lets run the post_execute method. This may result in a IgnoreOutputs Exception, which is
             # bubbled up to be handled at the callee layer.
             native_outputs = self.post_execute(new_user_params, native_outputs)
 
             # Short circuit the translation to literal map because what's returned may be a dj spec (or an
             # already-constructed LiteralMap if the dynamic task was a no-op), not python native values
+            # dynamic_execute returns a literal map in local execute so this also gets triggered.
             if isinstance(native_outputs, _literal_models.LiteralMap) or isinstance(
                 native_outputs, _dynamic_job.DynamicJobSpec
             ):
@@ -495,12 +539,28 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
                 py_type = self.get_type_for_output_var(k, v)
 
                 if isinstance(v, tuple):
-                    raise AssertionError(f"Output({k}) in task{self.name} received a tuple {v}, instead of {py_type}")
+                    raise TypeError(f"Output({k}) in task{self.name} received a tuple {v}, instead of {py_type}")
                 try:
                     literals[k] = TypeEngine.to_literal(exec_ctx, v, py_type, literal_type)
                 except Exception as e:
-                    logger.error(f"failed to convert return value for var {k} with error {type(e)}: {e}")
-                    raise e
+                    logger.error(f"Failed to convert return value for var {k} with error {type(e)}: {e}")
+                    raise TypeError(
+                        f"Failed to convert return value for var {k} for function {self.name} with error {type(e)}: {e}"
+                    ) from e
+
+            if self._disable_deck is False:
+                INPUT = "input"
+                OUTPUT = "output"
+
+                input_deck = Deck(INPUT)
+                for k, v in native_inputs.items():
+                    input_deck.append(TypeEngine.to_html(ctx, v, self.get_type_for_input_var(k, v)))
+
+                output_deck = Deck(OUTPUT)
+                for k, v in native_outputs_as_map.items():
+                    output_deck.append(TypeEngine.to_html(ctx, v, self.get_type_for_output_var(k, v)))
+
+                _output_deck(self.name.split(".")[-1], new_user_params)
 
             outputs_literal_map = _literal_models.LiteralMap(literals=literals)
             # After the execute has been successfully completed
@@ -542,6 +602,13 @@ class PythonTask(TrackedInstance, Task, Generic[T]):
         """
         return self._environment
 
+    @property
+    def disable_deck(self) -> bool:
+        """
+        If true, this task will not output deck html file
+        """
+        return self._disable_deck
+
 
 class TaskResolverMixin(object):
     """
