@@ -22,21 +22,27 @@ simple implementation that ships with the core.
 
 """
 
-import datetime
 import os
 import pathlib
+import re
+import shutil
+import sys
+import tempfile
 import typing
 from abc import abstractmethod
-from distutils import dir_util
 from shutil import copyfile
 from typing import Dict, Union
 from uuid import UUID
 
-from flytekit.common.exceptions.user import FlyteAssertion
-from flytekit.common.utils import PerformanceTimer
+from flytekit.configuration import DataConfig
+from flytekit.core.utils import PerformanceTimer
+from flytekit.exceptions.user import FlyteAssertion, FlyteValueException
 from flytekit.interfaces.random import random
 from flytekit.loggers import logger
 
+CURRENT_PYTHON = sys.version_info[:2]
+THREE_SEVEN = (3, 7)
+
 
 class UnsupportedPersistenceOp(Exception):
     """
@@ -135,13 +141,22 @@ class DataPersistencePlugins(object):
 
         cls._PLUGINS[protocol] = plugin
 
+    @staticmethod
+    def get_protocol(url: str):
+        # copy from fsspec https://github.com/fsspec/filesystem_spec/blob/fe09da6942ad043622212927df7442c104fe7932/fsspec/utils.py#L387-L391
+        parts = re.split(r"(\:\:|\://)", url, 1)
+        if len(parts) > 1:
+            return parts[0]
+        logger.info("Setting protocol to file")
+        return "file"
+
     @classmethod
     def find_plugin(cls, path: str) -> typing.Type[DataPersistence]:
         """
         Returns a plugin for the given protocol, else raise a TypeError
         """
         for k, p in cls._PLUGINS.items():
-            if path.startswith(k):
+            if cls.get_protocol(path) == k.replace("://", "") or path.startswith(k):
                 return p
         raise TypeError(f"No plugin found for matching protocol of path {path}")
 
@@ -160,6 +175,10 @@ class DataPersistencePlugins(object):
         """
         return protocol in cls._PLUGINS
 
+    @classmethod
+    def supported_protocols(cls) -> typing.List[str]:
+        return [k for k in cls._PLUGINS.keys()]
+
 
 class DiskPersistence(DataPersistence):
     """
@@ -206,17 +225,34 @@ class DiskPersistence(DataPersistence):
     def exists(self, path: str):
         return os.path.exists(self.strip_file_header(path))
 
+    def copy_tree(self, from_path: str, to_path: str):
+        # TODO: Remove this code after support for 3.7 is dropped and inline this function back
+        #    3.7 doesn't have dirs_exist_ok
+        if CURRENT_PYTHON == THREE_SEVEN:
+            tp = pathlib.Path(self.strip_file_header(to_path))
+            if tp.exists():
+                if not tp.is_dir():
+                    raise FlyteValueException(tp, f"Target {tp} exists but is not a dir")
+                files = os.listdir(tp)
+                if len(files) != 0:
+                    logger.debug(f"Deleting existing target dir {tp} with files {files}")
+                shutil.rmtree(tp)
+            shutil.copytree(self.strip_file_header(from_path), self.strip_file_header(to_path))
+        else:
+            # copytree will overwrite existing files in the to_path
+            shutil.copytree(self.strip_file_header(from_path), self.strip_file_header(to_path), dirs_exist_ok=True)
+
     def get(self, from_path: str, to_path: str, recursive: bool = False):
         if from_path != to_path:
             if recursive:
-                dir_util.copy_tree(self.strip_file_header(from_path), self.strip_file_header(to_path))
+                self.copy_tree(from_path, to_path)
             else:
                 copyfile(self.strip_file_header(from_path), self.strip_file_header(to_path))
 
     def put(self, from_path: str, to_path: str, recursive: bool = False):
         if from_path != to_path:
             if recursive:
-                dir_util.copy_tree(self.strip_file_header(from_path), self.strip_file_header(to_path))
+                self.copy_tree(from_path, to_path)
             else:
                 # Emulate s3's flat storage by automatically creating directory path
                 self._make_local_path(os.path.dirname(self.strip_file_header(to_path)))
@@ -231,13 +267,64 @@ class DiskPersistence(DataPersistence):
         return os.path.join(*args)
 
 
+def stringify_path(filepath):
+    """
+    Copied from `filesystem_spec <https://github.com/intake/filesystem_spec/blob/master/fsspec/utils.py#L287:5>`__
+
+    Attempt to convert a path-like object to a string.
+    Parameters
+    ----------
+    filepath: object to be converted
+    Returns
+    -------
+    filepath_str: maybe a string version of the object
+    Notes
+    -----
+    Objects supporting the fspath protocol (Python 3.6+) are coerced
+    according to its __fspath__ method.
+    For backwards compatibility with older Python version, pathlib.Path
+    objects are specially coerced.
+    Any other object is passed through unchanged, which includes bytes,
+    strings, buffers, or anything else that's not even path-like.
+    """
+    if isinstance(filepath, str):
+        return filepath
+    elif hasattr(filepath, "__fspath__"):
+        return filepath.__fspath__()
+    elif isinstance(filepath, pathlib.Path):
+        return str(filepath)
+    elif hasattr(filepath, "path"):
+        return filepath.path
+    else:
+        return filepath
+
+
+def split_protocol(urlpath):
+    """
+    Copied from `filesystem_spec <https://github.com/intake/filesystem_spec/blob/master/fsspec/core.py#L502>`__
+    Return protocol, path pair
+    """
+    urlpath = stringify_path(urlpath)
+    if "://" in urlpath:
+        protocol, path = urlpath.split("://", 1)
+        if len(protocol) > 1:
+            # excludes Windows paths
+            return protocol, path
+    return None, urlpath
+
+
 class FileAccessProvider(object):
     """
     This is the class that is available through the FlyteContext and can be used for persisting data to the remote
     durable store.
     """
 
-    def __init__(self, local_sandbox_dir: Union[str, os.PathLike], raw_output_prefix: str):
+    def __init__(
+        self,
+        local_sandbox_dir: Union[str, os.PathLike],
+        raw_output_prefix: str,
+        data_config: typing.Optional[DataConfig] = None,
+    ):
         """
         Args:
             local_sandbox_dir: A local temporary working directory, that should be used to store data
@@ -250,15 +337,29 @@ class FileAccessProvider(object):
         self._local_sandbox_dir.mkdir(parents=True, exist_ok=True)
         self._local = DiskPersistence(default_prefix=local_sandbox_dir_appended)
 
-        self._default_remote = DataPersistencePlugins.find_plugin(raw_output_prefix)(default_prefix=raw_output_prefix)
+        self._default_remote = DataPersistencePlugins.find_plugin(raw_output_prefix)(
+            default_prefix=raw_output_prefix, data_config=data_config
+        )
         self._raw_output_prefix = raw_output_prefix
+        self._data_config = data_config if data_config else DataConfig.auto()
+
+    @property
+    def raw_output_prefix(self) -> str:
+        return self._raw_output_prefix
+
+    @property
+    def data_config(self) -> DataConfig:
+        return self._data_config
 
     @staticmethod
     def is_remote(path: Union[str, os.PathLike]) -> bool:
         """
         Deprecated. Lets find a replacement
         """
-        return not (path.startswith("/") or path.startswith("file://"))
+        protocol, _ = split_protocol(path)
+        if protocol is None:
+            return False
+        return protocol != "file"
 
     @property
     def local_sandbox_dir(self) -> os.PathLike:
@@ -346,7 +447,11 @@ class FileAccessProvider(object):
         """
         try:
             with PerformanceTimer(f"Copying ({remote_path} -> {local_path})"):
-                DataPersistencePlugins.find_plugin(remote_path)().get(remote_path, local_path, recursive=is_multipart)
+                pathlib.Path(local_path).parent.mkdir(parents=True, exist_ok=True)
+                data_persistence_plugin = DataPersistencePlugins.find_plugin(remote_path)
+                data_persistence_plugin(data_config=self.data_config).get(
+                    remote_path, local_path, recursive=is_multipart
+                )
         except Exception as ex:
             raise FlyteAssertion(
                 f"Failed to get data from {remote_path} to {local_path} (recursive={is_multipart}).\n\n"
@@ -364,7 +469,9 @@ class FileAccessProvider(object):
         """
         try:
             with PerformanceTimer(f"Writing ({local_path} -> {remote_path})"):
-                DataPersistencePlugins.find_plugin(remote_path)().put(local_path, remote_path, recursive=is_multipart)
+                DataPersistencePlugins.find_plugin(remote_path)(data_config=self.data_config).put(
+                    local_path, remote_path, recursive=is_multipart
+                )
         except Exception as ex:
             raise FlyteAssertion(
                 f"Failed to put data from {local_path} to {remote_path} (recursive={is_multipart}).\n\n"
@@ -375,8 +482,9 @@ class FileAccessProvider(object):
 DataPersistencePlugins.register_plugin("file://", DiskPersistence)
 DataPersistencePlugins.register_plugin("/", DiskPersistence)
 
-# TODO make this use tmpdir
-tmp_dir = os.path.join("/tmp/flyte", datetime.datetime.now().strftime("%Y%m%d_%H%M%S"))
+flyte_tmp_dir = tempfile.mkdtemp(prefix="flyte-")
 default_local_file_access_provider = FileAccessProvider(
-    local_sandbox_dir=os.path.join(tmp_dir, "sandbox"), raw_output_prefix=os.path.join(tmp_dir, "raw")
+    local_sandbox_dir=os.path.join(flyte_tmp_dir, "sandbox"),
+    raw_output_prefix=os.path.join(flyte_tmp_dir, "raw"),
+    data_config=DataConfig.auto(),
 )
