@@ -1,14 +1,12 @@
 from __future__ import annotations
 
 import collections
-import typing
 from enum import Enum
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Set, Tuple, Union, cast
 
-from typing_extensions import Protocol
+from typing_extensions import Protocol, get_args
 
-from flytekit.common import constants as _common_constants
-from flytekit.common.exceptions import user as _user_exceptions
+from flytekit.core import constants as _common_constants
 from flytekit.core import context_manager as _flyte_context
 from flytekit.core import interface as flyte_interface
 from flytekit.core import type_engine
@@ -16,6 +14,7 @@ from flytekit.core.context_manager import BranchEvalMode, ExecutionState, FlyteC
 from flytekit.core.interface import Interface
 from flytekit.core.node import Node
 from flytekit.core.type_engine import DictTransformer, ListTransformer, TypeEngine
+from flytekit.exceptions import user as _user_exceptions
 from flytekit.models import interface as _interface_models
 from flytekit.models import literals as _literal_models
 from flytekit.models import literals as _literals_models
@@ -23,6 +22,7 @@ from flytekit.models import types as _type_models
 from flytekit.models import types as type_models
 from flytekit.models.core import workflow as _workflow_model
 from flytekit.models.literals import Primitive
+from flytekit.models.types import SimpleType
 
 
 def translate_inputs_to_literals(
@@ -64,33 +64,50 @@ def translate_inputs_to_literals(
     """
 
     def extract_value(
-        ctx: FlyteContext, input_val: Any, val_type: type, flyte_literal_type: _type_models.LiteralType
+        ctx: FlyteContext,
+        input_val: Any,
+        val_type: type,
+        flyte_literal_type: _type_models.LiteralType,
     ) -> _literal_models.Literal:
         if isinstance(input_val, list):
-            if flyte_literal_type.collection_type is None:
-                raise Exception(f"Not a collection type {flyte_literal_type} but got a list {input_val}")
+            lt = flyte_literal_type
+            python_type = val_type
+            if flyte_literal_type.union_type:
+                for i in range(len(flyte_literal_type.union_type.variants)):
+                    variant = flyte_literal_type.union_type.variants[i]
+                    if variant.collection_type:
+                        lt = variant
+                        python_type = get_args(val_type)[i]
+            if lt.collection_type is None:
+                raise TypeError(f"Not a collection type {flyte_literal_type} but got a list {input_val}")
             try:
-                sub_type = ListTransformer.get_sub_type(val_type)
+                sub_type = ListTransformer.get_sub_type(python_type)
             except ValueError:
                 if len(input_val) == 0:
                     raise
                 sub_type = type(input_val[0])
-            literals = [extract_value(ctx, v, sub_type, flyte_literal_type.collection_type) for v in input_val]
-            return _literal_models.Literal(collection=_literal_models.LiteralCollection(literals=literals))
+            literal_list = [extract_value(ctx, v, sub_type, lt.collection_type) for v in input_val]
+            return _literal_models.Literal(collection=_literal_models.LiteralCollection(literals=literal_list))
         elif isinstance(input_val, dict):
-            if (
-                flyte_literal_type.map_value_type is None
-                and flyte_literal_type.simple != _type_models.SimpleType.STRUCT
-            ):
-                raise Exception(f"Not a map type {flyte_literal_type} but got a map {input_val}")
-            k_type, sub_type = DictTransformer.get_dict_types(val_type)
-            if flyte_literal_type.simple == _type_models.SimpleType.STRUCT:
-                return TypeEngine.to_literal(ctx, input_val, type(input_val), flyte_literal_type)
+            lt = flyte_literal_type
+            python_type = val_type
+            if flyte_literal_type.union_type:
+                for i in range(len(flyte_literal_type.union_type.variants)):
+                    variant = flyte_literal_type.union_type.variants[i]
+                    if variant.map_value_type:
+                        lt = variant
+                        python_type = get_args(val_type)[i]
+                    if variant.simple == _type_models.SimpleType.STRUCT:
+                        lt = variant
+                        python_type = get_args(val_type)[i]
+            if lt.map_value_type is None and lt.simple != _type_models.SimpleType.STRUCT:
+                raise TypeError(f"Not a map type {lt} but got a map {input_val}")
+            if lt.simple == _type_models.SimpleType.STRUCT:
+                return TypeEngine.to_literal(ctx, input_val, type(input_val), lt)
             else:
-                literals = {
-                    k: extract_value(ctx, v, sub_type, flyte_literal_type.map_value_type) for k, v in input_val.items()
-                }
-                return _literal_models.Literal(map=_literal_models.LiteralMap(literals=literals))
+                k_type, sub_type = DictTransformer.get_dict_types(python_type)  # type: ignore
+                literal_map = {k: extract_value(ctx, v, sub_type, lt.map_value_type) for k, v in input_val.items()}
+                return _literal_models.Literal(map=_literal_models.LiteralMap(literals=literal_map))
         elif isinstance(input_val, Promise):
             # In the example above, this handles the "in2=a" type of argument
             return input_val.val
@@ -124,17 +141,16 @@ def translate_inputs_to_literals(
 
 
 def get_primitive_val(prim: Primitive) -> Any:
-    if prim.integer:
-        return prim.integer
-    if prim.datetime:
-        return prim.datetime
-    if prim.boolean:
-        return prim.boolean
-    if prim.duration:
-        return prim.duration
-    if prim.string_value:
-        return prim.string_value
-    return prim.float_value
+    for value in [
+        prim.integer,
+        prim.float_value,
+        prim.string_value,
+        prim.boolean,
+        prim.datetime,
+        prim.duration,
+    ]:
+        if value is not None:
+            return value
 
 
 class ConjunctionOps(Enum):
@@ -227,6 +243,9 @@ class ComparisonExpression(object):
     def __repr__(self):
         return f"Comp({self._lhs} {self._op.value} {self._rhs})"
 
+    def __str__(self):
+        return self.__repr__()
+
 
 class ConjunctionExpression(object):
     """
@@ -288,6 +307,9 @@ class ConjunctionExpression(object):
     def __repr__(self):
         return f"( {self._lhs} {self._op} {self._rhs} )"
 
+    def __str__(self):
+        return self.__repr__()
+
 
 # TODO: The NodeOutput object, which this Promise wraps, has an sdk_type. Since we're no longer using sdk types,
 #  we should consider adding a literal type to this object as well for downstream checking when Bindings are created.
@@ -325,6 +347,11 @@ class Promise(object):
     def __hash__(self):
         return hash(id(self))
 
+    def __rshift__(self, other: Union[Promise, VoidPromise]):
+        if not self.is_ready:
+            self.ref.node.runs_before(other.ref.node)
+        return other
+
     def with_var(self, new_var: str) -> Promise:
         if self.is_ready:
             return Promise(var=new_var, val=self.val)
@@ -414,7 +441,6 @@ class Promise(object):
     def with_overrides(self, *args, **kwargs):
         if not self.is_ready:
             # TODO, this should be forwarded, but right now this results in failure and we want to test this behavior
-            print(f"Forwarding to node {self.ref.node.id}")
             self.ref.node.with_overrides(*args, **kwargs)
         return self
 
@@ -428,7 +454,9 @@ class Promise(object):
 
 
 def create_native_named_tuple(
-    ctx: FlyteContext, promises: Union[Promise, typing.List[Promise]], entity_interface: Interface
+    ctx: FlyteContext,
+    promises: Optional[Union[Promise, List[Promise]]],
+    entity_interface: Interface,
 ) -> Optional[Tuple]:
     """
     Creates and returns a Named tuple with all variables that match the expected named outputs. this makes
@@ -442,8 +470,11 @@ def create_native_named_tuple(
         return None
 
     if isinstance(promises, Promise):
-        v = [v for k, v in entity_interface.outputs.items()][0]  # get output native type
-        return TypeEngine.to_python_value(ctx, promises.val, v)
+        k, v = [(k, v) for k, v in entity_interface.outputs.items()][0]  # get output native type
+        try:
+            return TypeEngine.to_python_value(ctx, promises.val, v)
+        except Exception as e:
+            raise AssertionError(f"Failed to convert value of output {k}, expected type {v}.") from e
 
     if len(promises) == 0:
         return None
@@ -459,7 +490,11 @@ def create_native_named_tuple(
                 "Workflow outputs can only be promises that are returned by tasks. Found a value of"
                 f"type {type(p)}. Workflows cannot return local variables or constants."
             )
-        outputs[p.var] = TypeEngine.to_python_value(ctx, p.val, entity_interface.outputs[p.var])
+        t = entity_interface.outputs[p.var]
+        try:
+            outputs[p.var] = TypeEngine.to_python_value(ctx, p.val, t)
+        except Exception as e:
+            raise AssertionError(f"Failed to convert value of output {p.var}, expected type {t}.") from e
 
     # Should this class be part of the Interface?
     t = collections.namedtuple(named_tuple_name, list(outputs.keys()))
@@ -468,7 +503,8 @@ def create_native_named_tuple(
 
 # To create a class that is a named tuple, we might have to create namedtuplemeta and manipulate the tuple
 def create_task_output(
-    promises: Optional[Union[List[Promise], Promise]], entity_interface: Optional[Interface] = None
+    promises: Optional[Union[List[Promise], Promise]],
+    entity_interface: Optional[Interface] = None,
 ) -> Optional[Union[Tuple[Promise], Promise]]:
     # TODO: Add VoidPromise here to simplify things at call site. Consider returning for [] below as well instead of
     #   raising an exception.
@@ -487,7 +523,7 @@ def create_task_output(
     if len(promises) == 1:
         if not entity_interface:
             return promises[0]
-        # See transform_signature_to_interface for more information, we're using the existence of a name as a proxy
+        # See transform_function_to_interface for more information, we're using the existence of a name as a proxy
         # for the user having specified a one-element typing.NamedTuple, which means we should _not_ extract it. We
         # should still return a tuple but it should be one of ours.
         if not entity_interface.output_tuple_name:
@@ -530,16 +566,26 @@ def create_task_output(
 
         def __rshift__(self, other: Any):
             # See comment for runs_before
-            return self
+            return other
 
-    return Output(*promises)
+    return Output(*promises)  # type: ignore
+
+
+def binding_from_flyte_std(
+    ctx: _flyte_context.FlyteContext,
+    var_name: str,
+    expected_literal_type: _type_models.LiteralType,
+    t_value: Any,
+) -> _literals_models.Binding:
+    binding_data = binding_data_from_python_std(ctx, expected_literal_type, t_value, t_value_type=None)
+    return _literals_models.Binding(var=var_name, binding=binding_data)
 
 
 def binding_data_from_python_std(
     ctx: _flyte_context.FlyteContext,
     expected_literal_type: _type_models.LiteralType,
-    t_value: typing.Any,
-    t_value_type: type,
+    t_value: Any,
+    t_value_type: Optional[type] = None,
 ) -> _literals_models.BindingData:
     # This handles the case where the given value is the output of another task
     if isinstance(t_value, Promise):
@@ -555,7 +601,7 @@ def binding_data_from_python_std(
         if expected_literal_type.collection_type is None:
             raise AssertionError(f"this should be a list and it is not: {type(t_value)} vs {expected_literal_type}")
 
-        sub_type = ListTransformer.get_sub_type(t_value_type)
+        sub_type = ListTransformer.get_sub_type(t_value_type) if t_value_type else None
         collection = _literals_models.BindingDataCollection(
             bindings=[
                 binding_data_from_python_std(ctx, expected_literal_type.collection_type, t, sub_type) for t in t_value
@@ -572,11 +618,14 @@ def binding_data_from_python_std(
             raise AssertionError(
                 f"this should be a Dictionary type and it is not: {type(t_value)} vs {expected_literal_type}"
             )
-        k_type, v_type = DictTransformer.get_dict_types(t_value_type)
         if expected_literal_type.simple == _type_models.SimpleType.STRUCT:
             lit = TypeEngine.to_literal(ctx, t_value, type(t_value), expected_literal_type)
             return _literals_models.BindingData(scalar=lit.scalar)
         else:
+            _, v_type = (
+                DictTransformer.get_dict_types(t_value_type) if t_value_type else None,
+                None,
+            )
             m = _literals_models.BindingDataMap(
                 bindings={
                     k: binding_data_from_python_std(ctx, expected_literal_type.map_value_type, v, v_type)
@@ -594,7 +643,7 @@ def binding_data_from_python_std(
         )
 
     # This is the scalar case - e.g. my_task(in1=5)
-    scalar = TypeEngine.to_literal(ctx, t_value, t_value_type, expected_literal_type).scalar
+    scalar = TypeEngine.to_literal(ctx, t_value, t_value_type or type(t_value), expected_literal_type).scalar
     return _literals_models.BindingData(scalar=scalar)
 
 
@@ -602,7 +651,7 @@ def binding_from_python_std(
     ctx: _flyte_context.FlyteContext,
     var_name: str,
     expected_literal_type: _type_models.LiteralType,
-    t_value: typing.Any,
+    t_value: Any,
     t_value_type: type,
 ) -> _literals_models.Binding:
     binding_data = binding_data_from_python_std(ctx, expected_literal_type, t_value, t_value_type)
@@ -619,8 +668,9 @@ class VoidPromise(object):
     VoidPromise cannot be interacted with and does not allow comparisons or any operations
     """
 
-    def __init__(self, task_name: str):
+    def __init__(self, task_name: str, ref: Optional[NodeOutput] = None):
         self._task_name = task_name
+        self._ref = ref
 
     def runs_before(self, *args, **kwargs):
         """
@@ -628,8 +678,19 @@ class VoidPromise(object):
         where a task returns nothing.
         """
 
-    def __rshift__(self, *args, **kwargs):
-        ...  # See runs_before
+    @property
+    def ref(self) -> Optional[NodeOutput]:
+        return self._ref
+
+    def __rshift__(self, other: Union[Promise, VoidPromise]):
+        if self.ref:
+            self.ref.node.runs_before(other.ref.node)
+        return other
+
+    def with_overrides(self, *args, **kwargs):
+        if self.ref:
+            self.ref.node.with_overrides(*args, **kwargs)
+        return self
 
     @property
     def task_name(self):
@@ -690,7 +751,8 @@ class NodeOutput(type_models.OutputReference):
     @property
     def node_id(self):
         """
-        Override the underlying node_id property to refer to SdkNode.
+        Override the underlying node_id property to refer to the Node's id. This is to make sure that overriding
+        node IDs from with_overrides gets serialized correctly.
         :rtype: Text
         """
         return self.node.id
@@ -718,13 +780,152 @@ class SupportsNodeCreation(Protocol):
         ...
 
 
+class HasFlyteInterface(Protocol):
+    @property
+    def name(self) -> str:
+        ...
+
+    @property
+    def interface(self) -> _interface_models.TypedInterface:
+        ...
+
+    def construct_node_metadata(self) -> _workflow_model.NodeMetadata:
+        ...
+
+
+def extract_obj_name(name: str) -> str:
+    """
+    Generates a shortened name, without the module information. Useful for node-names etc. Only extracts the final
+    object information often separated by `.` in the python fully qualified notation
+    """
+    if name is None:
+        return ""
+    if "." in name:
+        return name.split(".")[-1]
+    return name
+
+
+def create_and_link_node_from_remote(
+    ctx: FlyteContext,
+    entity: HasFlyteInterface,
+    _inputs_not_allowed: Optional[Set[str]] = None,
+    _ignorable_inputs: Optional[Set[str]] = None,
+    **kwargs,
+) -> Optional[Union[Tuple[Promise], Promise, VoidPromise]]:
+    """
+    This method is used to generate a node with bindings especially when using remote entities, like FlyteWorkflow,
+    FlyteTask and FlyteLaunchplan.
+
+    This method is kept separate from the similar named method `create_and_link_node` as remote entities have to be
+    handled differently. The major difference arises from the fact that the remote entities do not have a python
+    interface, so all comparisons need to happen using the Literals.
+
+    :param ctx: FlyteContext
+    :param entity: RemoteEntity
+    :param _inputs_not_allowed: Set of all variable names that should not be provided when using this entity.
+                     Useful for Launchplans with `fixed` inputs
+    :param _ignorable_inputs: Set of all variable names that are optional, but if provided will be overriden. Useful
+                     for launchplans with `default` inputs
+    :param kwargs: Dict[str, Any] default inputs passed from the user to this entity. Can be promises.
+    :return:  Optional[Union[Tuple[Promise], Promise, VoidPromise]]
+    """
+    if ctx.compilation_state is None:
+        raise _user_exceptions.FlyteAssertion("Cannot create node when not compiling...")
+
+    used_inputs = set()
+    bindings = []
+
+    typed_interface = entity.interface
+
+    if _inputs_not_allowed:
+        inputs_not_allowed_specified = _inputs_not_allowed.intersection(kwargs.keys())
+        if inputs_not_allowed_specified:
+            raise _user_exceptions.FlyteAssertion(
+                f"Fixed inputs cannot be specified. Please remove the following inputs - {inputs_not_allowed_specified}"
+            )
+
+    for k in sorted(typed_interface.inputs):
+        var = typed_interface.inputs[k]
+        if k not in kwargs:
+            if _inputs_not_allowed and _ignorable_inputs:
+                if k in _ignorable_inputs or k in _inputs_not_allowed:
+                    continue
+            # TODO to improve the error message, should we show python equivalent types for var.type?
+            raise _user_exceptions.FlyteAssertion("Missing input `{}` type `{}`".format(k, var.type))
+        v = kwargs[k]
+        # This check ensures that tuples are not passed into a function, as tuples are not supported by Flyte
+        # Usually a Tuple will indicate that multiple outputs from a previous task were accidentally passed
+        # into the function.
+        if isinstance(v, tuple):
+            raise AssertionError(
+                f"Variable({k}) for function({entity.name}) cannot receive a multi-valued tuple {v}."
+                f" Check if the predecessor function returning more than one value?"
+            )
+        try:
+            bindings.append(
+                binding_from_flyte_std(
+                    ctx,
+                    var_name=k,
+                    expected_literal_type=var.type,
+                    t_value=v,
+                )
+            )
+            used_inputs.add(k)
+        except Exception as e:
+            raise AssertionError(f"Failed to Bind variable {k} for function {entity.name}.") from e
+
+    extra_inputs = used_inputs ^ set(kwargs.keys())
+    if len(extra_inputs) > 0:
+        raise _user_exceptions.FlyteAssertion(
+            f"Too many inputs for [{entity.name}] Expected inputs: {typed_interface.inputs.keys()} "
+            f"- extra inputs: {extra_inputs}"
+        )
+
+    # Detect upstream nodes
+    # These will be our core Nodes until we can amend the Promise to use NodeOutputs that reference our Nodes
+    upstream_nodes = list(
+        set(
+            [
+                input_val.ref.node
+                for input_val in kwargs.values()
+                if isinstance(input_val, Promise) and input_val.ref.node_id != _common_constants.GLOBAL_INPUT_NODE_ID
+            ]
+        )
+    )
+
+    flytekit_node = Node(
+        id=f"{ctx.compilation_state.prefix}n{len(ctx.compilation_state.nodes)}",
+        metadata=entity.construct_node_metadata(),
+        bindings=sorted(bindings, key=lambda b: b.var),
+        upstream_nodes=upstream_nodes,
+        flyte_entity=entity,
+    )
+    ctx.compilation_state.add_node(flytekit_node)
+
+    if len(typed_interface.outputs) == 0:
+        return VoidPromise(entity.name, NodeOutput(node=flytekit_node, var="placeholder"))
+
+    # Create a node output object for each output, they should all point to this node of course.
+    node_outputs = []
+    for output_name, output_var_model in typed_interface.outputs.items():
+        node_outputs.append(Promise(output_name, NodeOutput(node=flytekit_node, var=output_name)))
+
+    return create_task_output(node_outputs)
+
+
 def create_and_link_node(
     ctx: FlyteContext,
     entity: SupportsNodeCreation,
     **kwargs,
-):
+) -> Optional[Union[Tuple[Promise], Promise, VoidPromise]]:
     """
-    This method is used to generate a node with bindings. This is not used in the execution path.
+    This method is used to generate a node with bindings within a flytekit workflow. this is useful to traverse the
+    workflow using regular python interpreter and generate nodes and promises whenever an execution is encountered
+
+    :param ctx: FlyteContext
+    :param entity: RemoteEntity
+    :param kwargs: Dict[str, Any] default inputs passed from the user to this entity. Can be promises.
+    :return:  Optional[Union[Tuple[Promise], Promise, VoidPromise]]
     """
     if ctx.compilation_state is None:
         raise _user_exceptions.FlyteAssertion("Cannot create node when not compiling...")
@@ -740,7 +941,20 @@ def create_and_link_node(
     for k in sorted(interface.inputs):
         var = typed_interface.inputs[k]
         if k not in kwargs:
-            raise _user_exceptions.FlyteAssertion("Input was not specified for: {} of type {}".format(k, var.type))
+            is_optional = False
+            if var.type.union_type:
+                for variant in var.type.union_type.variants:
+                    if variant.simple == SimpleType.NONE:
+                        val, _default = interface.inputs_with_defaults[k]
+                        if _default is not None:
+                            raise ValueError(
+                                f"The default value for the optional type must be None, but got {_default}"
+                            )
+                        is_optional = True
+            if not is_optional:
+                raise _user_exceptions.FlyteAssertion("Input was not specified for: {} of type {}".format(k, var.type))
+            else:
+                continue
         v = kwargs[k]
         # This check ensures that tuples are not passed into a function, as tuples are not supported by Flyte
         # Usually a Tuple will indicate that multiple outputs from a previous task were accidentally passed
@@ -750,12 +964,19 @@ def create_and_link_node(
                 f"Variable({k}) for function({entity.name}) cannot receive a multi-valued tuple {v}."
                 f" Check if the predecessor function returning more than one value?"
             )
-        bindings.append(
-            binding_from_python_std(
-                ctx, var_name=k, expected_literal_type=var.type, t_value=v, t_value_type=interface.inputs[k]
+        try:
+            bindings.append(
+                binding_from_python_std(
+                    ctx,
+                    var_name=k,
+                    expected_literal_type=var.type,
+                    t_value=v,
+                    t_value_type=interface.inputs[k],
+                )
             )
-        )
-        used_inputs.add(k)
+            used_inputs.add(k)
+        except Exception as e:
+            raise AssertionError(f"Failed to Bind variable {k} for function {entity.name}.") from e
 
     extra_inputs = used_inputs ^ set(kwargs.keys())
     if len(extra_inputs) > 0:
@@ -786,13 +1007,11 @@ def create_and_link_node(
     ctx.compilation_state.add_node(flytekit_node)
 
     if len(typed_interface.outputs) == 0:
-        return VoidPromise(entity.name)
+        return VoidPromise(entity.name, NodeOutput(node=flytekit_node, var="placeholder"))
 
     # Create a node output object for each output, they should all point to this node of course.
     node_outputs = []
     for output_name, output_var_model in typed_interface.outputs.items():
-        # TODO: If node id gets updated later, we have to make sure to update the NodeOutput model's ID, which
-        #  is currently just a static str
         node_outputs.append(Promise(output_name, NodeOutput(node=flytekit_node, var=output_name)))
         # Don't print this, it'll crash cuz sdk_node._upstream_node_ids might be None, but idl code will break
 
@@ -804,7 +1023,7 @@ class LocallyExecutable(Protocol):
         ...
 
 
-def flyte_entity_call_handler(entity: Union[SupportsNodeCreation, LocallyExecutable], *args, **kwargs):
+def flyte_entity_call_handler(entity: SupportsNodeCreation, *args, **kwargs):
     """
     This function is the call handler for tasks, workflows, and launch plans (which redirects to the underlying
     workflow). The logic is the same for all three, but we did not want to create base class, hence this separate
@@ -828,8 +1047,10 @@ def flyte_entity_call_handler(entity: Union[SupportsNodeCreation, LocallyExecuta
         )
     # Make sure arguments are part of interface
     for k, v in kwargs.items():
-        if k not in entity.python_interface.inputs:
-            raise ValueError(f"Received unexpected keyword argument {k}")
+        if k not in cast(SupportsNodeCreation, entity).python_interface.inputs:
+            raise ValueError(
+                f"Received unexpected keyword argument {k} in function {cast(SupportsNodeCreation, entity).name}"
+            )
 
     ctx = FlyteContextManager.current_context()
 
@@ -837,34 +1058,39 @@ def flyte_entity_call_handler(entity: Union[SupportsNodeCreation, LocallyExecuta
         return create_and_link_node(ctx, entity=entity, **kwargs)
     elif ctx.execution_state is not None and ctx.execution_state.mode == ExecutionState.Mode.LOCAL_WORKFLOW_EXECUTION:
         if ctx.execution_state.branch_eval_mode == BranchEvalMode.BRANCH_SKIPPED:
-            if len(entity.python_interface.inputs) > 0 or len(entity.python_interface.outputs) > 0:
-                output_names = list(entity.python_interface.outputs.keys())
+            if (
+                len(cast(SupportsNodeCreation, entity).python_interface.inputs) > 0
+                or len(cast(SupportsNodeCreation, entity).python_interface.outputs) > 0
+            ):
+                output_names = list(cast(SupportsNodeCreation, entity).python_interface.outputs.keys())
                 if len(output_names) == 0:
                     return VoidPromise(entity.name)
                 vals = [Promise(var, None) for var in output_names]
-                return create_task_output(vals, entity.python_interface)
+                return create_task_output(vals, cast(SupportsNodeCreation, entity).python_interface)
             else:
                 return None
-        return entity.local_execute(ctx, **kwargs)
+        return cast(LocallyExecutable, entity).local_execute(ctx, **kwargs)
     else:
         with FlyteContextManager.with_context(
             ctx.with_execution_state(
                 ctx.new_execution_state().with_params(mode=ExecutionState.Mode.LOCAL_WORKFLOW_EXECUTION)
             )
         ) as child_ctx:
-            result = entity.local_execute(child_ctx, **kwargs)
+            cast(FlyteContext, child_ctx).user_space_params._decks = []
+            result = cast(LocallyExecutable, entity).local_execute(child_ctx, **kwargs)
 
-        expected_outputs = len(entity.python_interface.outputs)
+        expected_outputs = len(cast(SupportsNodeCreation, entity).python_interface.outputs)
         if expected_outputs == 0:
             if result is None or isinstance(result, VoidPromise):
                 return None
             else:
-                raise Exception(f"Workflow local execution expected 0 outputs but something received {result}")
+                raise Exception(f"Received an output when workflow local execution expected None. Received: {result}")
 
         if (1 < expected_outputs == len(result)) or (result is not None and expected_outputs == 1):
-            return create_native_named_tuple(ctx, result, entity.python_interface)
+            return create_native_named_tuple(ctx, result, cast(SupportsNodeCreation, entity).python_interface)
 
         raise ValueError(
-            f"Expected outputs and actual outputs do not match. Result {result}. "
-            f"Python interface: {entity.python_interface}"
+            f"Expected outputs and actual outputs do not match."
+            f"Result {result}. "
+            f"Python interface: {cast(SupportsNodeCreation, entity).python_interface}"
         )
