@@ -14,25 +14,26 @@
 """
 
 
-import inspect
 from abc import ABC
 from collections import OrderedDict
 from enum import Enum
 from typing import Any, Callable, List, Optional, TypeVar, Union
 
-from flytekit.common.exceptions import scopes as exception_scopes
 from flytekit.core.base_task import Task, TaskResolverMixin
-from flytekit.core.context_manager import ExecutionState, FastSerializationSettings, FlyteContext, FlyteContextManager
+from flytekit.core.context_manager import ExecutionState, FlyteContext, FlyteContextManager
 from flytekit.core.docstring import Docstring
-from flytekit.core.interface import transform_signature_to_interface
+from flytekit.core.interface import transform_function_to_interface
+from flytekit.core.promise import VoidPromise, translate_inputs_to_literals
 from flytekit.core.python_auto_container import PythonAutoContainerTask, default_task_resolver
-from flytekit.core.tracker import isnested, istestfunction
+from flytekit.core.tracker import extract_task_module, is_functools_wrapped_module_level, isnested, istestfunction
 from flytekit.core.workflow import (
     PythonFunctionWorkflow,
     WorkflowFailurePolicy,
     WorkflowMetadata,
     WorkflowMetadataDefaults,
 )
+from flytekit.exceptions import scopes as exception_scopes
+from flytekit.exceptions.user import FlyteValueException
 from flytekit.loggers import logger
 from flytekit.models import dynamic_job as _dynamic_job
 from flytekit.models import literals as _literal_models
@@ -99,7 +100,7 @@ class PythonFunctionTask(PythonAutoContainerTask[T]):
         task_function: Callable,
         task_type="python-task",
         ignore_input_vars: Optional[List[str]] = None,
-        execution_mode: Optional[ExecutionBehavior] = ExecutionBehavior.DEFAULT,
+        execution_mode: ExecutionBehavior = ExecutionBehavior.DEFAULT,
         task_resolver: Optional[TaskResolverMixin] = None,
         **kwargs,
     ):
@@ -110,17 +111,16 @@ class PythonFunctionTask(PythonAutoContainerTask[T]):
                                   can be used to inject some client side variables only. Prefer using ExecutionParams
         :param Optional[ExecutionBehavior] execution_mode: Defines how the execution should behave, for example
             executing normally or specially handling a dynamic case.
-        :param Optional[TaskResolverMixin] task_type: String task type to be associated with this Task
+        :param str task_type: String task type to be associated with this Task
         """
         if task_function is None:
             raise ValueError("TaskFunction is a required parameter for PythonFunctionTask")
-        self._native_interface = transform_signature_to_interface(
-            inspect.signature(task_function), Docstring(callable_=task_function)
-        )
+        self._native_interface = transform_function_to_interface(task_function, Docstring(callable_=task_function))
         mutated_interface = self._native_interface.remove_inputs(ignore_input_vars)
+        name, _, _, _ = extract_task_module(task_function)
         super().__init__(
             task_type=task_type,
-            name=f"{task_function.__module__}.{task_function.__name__}",
+            name=name,
             interface=mutated_interface,
             task_config=task_config,
             task_resolver=task_resolver,
@@ -130,15 +130,22 @@ class PythonFunctionTask(PythonAutoContainerTask[T]):
         if self._task_resolver is default_task_resolver:
             # The default task resolver can't handle nested functions
             # TODO: Consider moving this to a can_handle function or something inside the resolver itself.
-            if not istestfunction(func=task_function) and isnested(func=task_function):
+            if (
+                not istestfunction(func=task_function)
+                and isnested(func=task_function)
+                and not is_functools_wrapped_module_level(task_function)
+            ):
                 raise ValueError(
                     "TaskFunction cannot be a nested/inner or local function. "
                     "It should be accessible at a module level for Flyte to execute it. Test modules with "
-                    "names beginning with `test_` are allowed to have nested tasks. If you want to create your own tasks"
-                    "use the TaskResolverMixin"
+                    "names beginning with `test_` are allowed to have nested tasks. "
+                    "If you're decorating your task function with custom decorators, use functools.wraps "
+                    "or functools.update_wrapper on the function wrapper. "
+                    "Alternatively if you want to create your own tasks with custom behavior use the TaskResolverMixin"
                 )
         self._task_function = task_function
         self._execution_mode = execution_mode
+        self._wf = None  # For dynamic tasks
 
     @property
     def execution_mode(self) -> ExecutionBehavior:
@@ -158,6 +165,14 @@ class PythonFunctionTask(PythonAutoContainerTask[T]):
         elif self.execution_mode == self.ExecutionBehavior.DYNAMIC:
             return self.dynamic_execute(self._task_function, **kwargs)
 
+    def _create_and_cache_dynamic_workflow(self):
+        if self._wf is None:
+            workflow_meta = WorkflowMetadata(on_failure=WorkflowFailurePolicy.FAIL_IMMEDIATELY)
+            defaults = WorkflowMetadataDefaults(
+                interruptible=self.metadata.interruptible if self.metadata.interruptible is not None else False
+            )
+            self._wf = PythonFunctionWorkflow(self._task_function, metadata=workflow_meta, default_metadata=defaults)
+
     def compile_into_workflow(
         self, ctx: FlyteContext, task_function: Callable, **kwargs
     ) -> Union[_dynamic_job.DynamicJobSpec, _literal_models.LiteralMap]:
@@ -169,20 +184,15 @@ class PythonFunctionTask(PythonAutoContainerTask[T]):
         from flytekit.core.task import ReferenceTask
 
         if not ctx.compilation_state:
-            cs = ctx.new_compilation_state("dynamic")
+            cs = ctx.new_compilation_state(prefix="d")
         else:
-            cs = ctx.compilation_state.with_params(prefix="dynamic")
+            cs = ctx.compilation_state.with_params(prefix="d")
 
         with FlyteContextManager.with_context(ctx.with_compilation_state(cs)):
             # TODO: Resolve circular import
-            from flytekit.common.translator import get_serializable
+            from flytekit.tools.translator import get_serializable
 
-            workflow_metadata = WorkflowMetadata(on_failure=WorkflowFailurePolicy.FAIL_IMMEDIATELY)
-            defaults = WorkflowMetadataDefaults(
-                interruptible=self.metadata.interruptible if self.metadata.interruptible is not None else False
-            )
-
-            self._wf = PythonFunctionWorkflow(task_function, metadata=workflow_metadata, default_metadata=defaults)
+            self._create_and_cache_dynamic_workflow()
             self._wf.compile(**kwargs)
 
             wf = self._wf
@@ -207,7 +217,7 @@ class PythonFunctionTask(PythonAutoContainerTask[T]):
             for entity, model in model_entities.items():
                 # We only care about gathering tasks here. Launch plans are handled by
                 # propeller. Subworkflows should already be in the workflow spec.
-                if not isinstance(entity, Task):
+                if not isinstance(entity, Task) and not isinstance(entity, task_models.TaskSpec):
                     continue
 
                 # We are currently not supporting reference tasks since these will
@@ -225,29 +235,6 @@ class PythonFunctionTask(PythonAutoContainerTask[T]):
                 # DynamicJobSpec later
                 tts.append(model.template)
 
-            if ctx.serialization_settings.should_fast_serialize():
-                if (
-                    not ctx.execution_state
-                    or not ctx.execution_state.additional_context
-                    or not ctx.execution_state.additional_context.get("dynamic_addl_distro")
-                ):
-                    raise AssertionError(
-                        "Compilation for a dynamic workflow called in fast execution mode but no additional code "
-                        "distribution could be retrieved"
-                    )
-                logger.warn(f"ctx.execution_state.additional_context {ctx.execution_state.additional_context}")
-                for task_template in tts:
-                    sanitized_args = []
-                    for arg in task_template.container.args:
-                        if arg == "{{ .remote_package_path }}":
-                            sanitized_args.append(ctx.execution_state.additional_context.get("dynamic_addl_distro"))
-                        elif arg == "{{ .dest_dir }}":
-                            sanitized_args.append(ctx.execution_state.additional_context.get("dynamic_dest_dir", "."))
-                        else:
-                            sanitized_args.append(arg)
-                    del task_template.container.args[:]
-                    task_template.container.args.extend(sanitized_args)
-
             dj_spec = _dynamic_job.DynamicJobSpec(
                 min_successes=len(workflow_spec.template.nodes),
                 tasks=tts,
@@ -271,26 +258,46 @@ class PythonFunctionTask(PythonAutoContainerTask[T]):
         representing that newly generated workflow, instead of executing it.
         """
         ctx = FlyteContextManager.current_context()
-
         if ctx.execution_state and ctx.execution_state.mode == ExecutionState.Mode.LOCAL_WORKFLOW_EXECUTION:
-            updated_exec_state = ctx.execution_state.with_params(mode=ExecutionState.Mode.TASK_EXECUTION)
-            with FlyteContextManager.with_context(ctx.with_execution_state(updated_exec_state)):
-                logger.info("Executing Dynamic workflow, using raw inputs")
-                return exception_scopes.user_entry_point(task_function)(**kwargs)
+            # The rest of this function mimics the local_execute of the workflow. We can't use the workflow
+            # local_execute directly though since that converts inputs into Promises.
+            logger.debug(f"Executing Dynamic workflow, using raw inputs {kwargs}")
+            self._create_and_cache_dynamic_workflow()
+            function_outputs = self._wf.execute(**kwargs)
+
+            if isinstance(function_outputs, VoidPromise) or function_outputs is None:
+                return VoidPromise(self.name)
+
+            if len(self._wf.python_interface.outputs) == 0:
+                raise FlyteValueException(function_outputs, "Interface output should've been VoidPromise or None.")
+
+            # TODO: This will need to be cleaned up when we revisit top-level tuple support.
+            expected_output_names = list(self.python_interface.outputs.keys())
+            if len(expected_output_names) == 1:
+                # Here we have to handle the fact that the wf could've been declared with a typing.NamedTuple of
+                # length one. That convention is used for naming outputs - and single-length-NamedTuples are
+                # particularly troublesome but elegant handling of them is not a high priority
+                # Again, we're using the output_tuple_name as a proxy.
+                if self.python_interface.output_tuple_name and isinstance(function_outputs, tuple):
+                    wf_outputs_as_map = {expected_output_names[0]: function_outputs[0]}
+                else:
+                    wf_outputs_as_map = {expected_output_names[0]: function_outputs}
+            else:
+                wf_outputs_as_map = {
+                    expected_output_names[i]: function_outputs[i] for i, _ in enumerate(function_outputs)
+                }
+
+            # In a normal workflow, we'd repackage the promises coming from tasks into new Promises matching the
+            # workflow's interface. For a dynamic workflow, just return the literal map.
+            wf_outputs_as_literal_dict = translate_inputs_to_literals(
+                ctx,
+                wf_outputs_as_map,
+                flyte_interface_types=self.interface.outputs,
+                native_types=self.python_interface.outputs,
+            )
+            return _literal_models.LiteralMap(literals=wf_outputs_as_literal_dict)
 
         if ctx.execution_state and ctx.execution_state.mode == ExecutionState.Mode.TASK_EXECUTION:
-            is_fast_execution = bool(
-                ctx.execution_state
-                and ctx.execution_state.additional_context
-                and ctx.execution_state.additional_context.get("dynamic_addl_distro")
-            )
-            if is_fast_execution:
-                ctx = ctx.with_serialization_settings(
-                    ctx.serialization_settings.new_builder()
-                    .with_fast_serialization_settings(FastSerializationSettings(enabled=True))
-                    .build()
-                )
-
             return self.compile_into_workflow(ctx, task_function, **kwargs)
 
         if ctx.execution_state and ctx.execution_state.mode == ExecutionState.Mode.LOCAL_TASK_EXECUTION:
