@@ -1,13 +1,11 @@
 from __future__ import annotations
 
-import inspect
 from dataclasses import dataclass
 from enum import Enum
+from functools import update_wrapper
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
-from flytekit.common import constants as _common_constants
-from flytekit.common.exceptions import scopes as exception_scopes
-from flytekit.common.exceptions.user import FlyteValidationException, FlyteValueException
+from flytekit.core import constants as _common_constants
 from flytekit.core.base_task import PythonTask
 from flytekit.core.class_based_resolver import ClassStorageTaskResolver
 from flytekit.core.condition import ConditionalSection
@@ -15,9 +13,9 @@ from flytekit.core.context_manager import CompilationState, FlyteContext, FlyteC
 from flytekit.core.docstring import Docstring
 from flytekit.core.interface import (
     Interface,
+    transform_function_to_interface,
     transform_inputs_to_parameters,
     transform_interface_to_typed_interface,
-    transform_signature_to_interface,
 )
 from flytekit.core.launch_plan import LaunchPlan
 from flytekit.core.node import Node
@@ -27,16 +25,21 @@ from flytekit.core.promise import (
     VoidPromise,
     binding_from_python_std,
     create_task_output,
+    extract_obj_name,
     flyte_entity_call_handler,
     translate_inputs_to_literals,
 )
 from flytekit.core.python_auto_container import PythonAutoContainerTask
 from flytekit.core.reference_entity import ReferenceEntity, WorkflowReference
+from flytekit.core.tracker import extract_task_module
 from flytekit.core.type_engine import TypeEngine
+from flytekit.exceptions import scopes as exception_scopes
+from flytekit.exceptions.user import FlyteValidationException, FlyteValueException
 from flytekit.loggers import logger
 from flytekit.models import interface as _interface_models
 from flytekit.models import literals as _literal_models
 from flytekit.models.core import workflow as _workflow_model
+from flytekit.models.documentation import Description, Documentation
 
 GLOBAL_START_NODE = Node(
     id=_common_constants.GLOBAL_INPUT_NODE_ID,
@@ -132,7 +135,7 @@ def get_promise(binding_data: _literal_models.BindingData, outputs_cache: Dict[N
             val=_literal_models.Literal(collection=_literal_models.LiteralCollection(literals=literals)),
         )
     elif binding_data.map is not None:
-        literals = {}
+        literals = {}  # type: ignore
         for k, bd in binding_data.map.bindings.items():
             p = get_promise(bd, outputs_cache)
             literals[k] = p.val
@@ -166,6 +169,7 @@ class WorkflowBase(object):
         workflow_metadata: WorkflowMetadata,
         workflow_metadata_defaults: WorkflowMetadataDefaults,
         python_interface: Interface,
+        docs: Optional[Documentation] = None,
         **kwargs,
     ):
         self._name = name
@@ -176,7 +180,21 @@ class WorkflowBase(object):
         self._inputs = {}
         self._unbound_inputs = set()
         self._nodes = []
-        self._output_bindings: Optional[List[_literal_models.Binding]] = []
+        self._output_bindings: List[_literal_models.Binding] = []
+        self._docs = docs
+
+        if self._python_interface.docstring:
+            if self.docs is None:
+                self._docs = Documentation(
+                    short_description=self._python_interface.docstring.short_description,
+                    long_description=Description(value=self._python_interface.docstring.long_description),
+                )
+            else:
+                if self._python_interface.docstring.short_description:
+                    self._docs.short_description = self._python_interface.docstring.short_description
+                if self._python_interface.docstring.long_description:
+                    self._docs = Description(value=self._python_interface.docstring.long_description)
+
         FlyteEntities.entities.append(self)
         super().__init__(**kwargs)
 
@@ -184,9 +202,13 @@ class WorkflowBase(object):
     def name(self) -> str:
         return self._name
 
+    @property
+    def docs(self):
+        return self._docs
+
     @property
     def short_name(self) -> str:
-        return self._name.split(".")[-1]
+        return extract_obj_name(self._name)
 
     @property
     def workflow_metadata(self) -> Optional[WorkflowMetadata]:
@@ -222,7 +244,7 @@ class WorkflowBase(object):
 
     def construct_node_metadata(self) -> _workflow_model.NodeMetadata:
         return _workflow_model.NodeMetadata(
-            name=f"{self.__module__}.{self.name}",
+            name=extract_obj_name(self.name),
             interruptible=self.workflow_metadata_defaults.interruptible,
         )
 
@@ -238,7 +260,7 @@ class WorkflowBase(object):
     def execute(self, **kwargs):
         raise Exception("Should not be called")
 
-    def local_execute(self, ctx: FlyteContext, **kwargs) -> Union[Tuple[Promise], Promise, VoidPromise]:
+    def local_execute(self, ctx: FlyteContext, **kwargs) -> Union[Tuple[Promise], Promise, VoidPromise, None]:
         # This is done to support the invariant that Workflow local executions always work with Promise objects
         # holding Flyte literal values. Even in a wf, a user can call a sub-workflow with a Python native value.
         for k, v in kwargs.items():
@@ -262,15 +284,13 @@ class WorkflowBase(object):
             if len(self.python_interface.outputs) != 0:
                 raise FlyteValueException(
                     function_outputs,
-                    f"{function_outputs} received but interface has {len(self.python_interface.outputs)} outputs.",
+                    f"Interface has {len(self.python_interface.outputs)} outputs.",
                 )
             return VoidPromise(self.name)
 
         # Because we should've already returned in the above check, we just raise an error here.
         if len(self.python_interface.outputs) == 0:
-            raise FlyteValueException(
-                function_outputs, f"{function_outputs} received but should've been VoidPromise or None."
-            )
+            raise FlyteValueException(function_outputs, "Interface output should've been VoidPromise or None.")
 
         expected_output_names = list(self.python_interface.outputs.keys())
         if len(expected_output_names) == 1:
@@ -418,7 +438,7 @@ class ImperativeWorkflow(WorkflowBase):
 
             # Because we should've already returned in the above check, we just raise an Exception here.
             if len(entity.python_interface.outputs) == 0:
-                raise FlyteValueException(results, f"{results} received but should've been VoidPromise or None.")
+                raise FlyteValueException(results, "Interface output should've been VoidPromise or None.")
 
             # if there's only one output,
             if len(expected_output_names) == 1:
@@ -487,7 +507,7 @@ class ImperativeWorkflow(WorkflowBase):
             for input_value in filter(lambda x: isinstance(x, Promise), all_input_values):
                 if input_value in self._unbound_inputs:
                     self._unbound_inputs.remove(input_value)
-            return n
+            return n  # type: ignore
 
     def add_workflow_input(self, input_name: str, python_type: Type) -> Interface:
         """
@@ -571,11 +591,12 @@ class PythonFunctionWorkflow(WorkflowBase, ClassStorageTaskResolver):
         workflow_function: Callable,
         metadata: Optional[WorkflowMetadata],
         default_metadata: Optional[WorkflowMetadataDefaults],
-        docstring: Docstring = None,
+        docstring: Optional[Docstring] = None,
+        docs: Optional[Documentation] = None,
     ):
-        name = f"{workflow_function.__module__}.{workflow_function.__name__}"
+        name, _, _, _ = extract_task_module(workflow_function)
         self._workflow_function = workflow_function
-        native_interface = transform_signature_to_interface(inspect.signature(workflow_function), docstring=docstring)
+        native_interface = transform_function_to_interface(workflow_function, docstring=docstring)
 
         # TODO do we need this - can this not be in launchplan only?
         #    This can be in launch plan only, but is here only so that we don't have to re-evaluate. Or
@@ -586,6 +607,7 @@ class PythonFunctionWorkflow(WorkflowBase, ClassStorageTaskResolver):
             workflow_metadata=metadata,
             workflow_metadata_defaults=default_metadata,
             python_interface=native_interface,
+            docs=docs,
         )
 
     @property
@@ -603,7 +625,7 @@ class PythonFunctionWorkflow(WorkflowBase, ClassStorageTaskResolver):
         ctx = FlyteContextManager.current_context()
         self._input_parameters = transform_inputs_to_parameters(ctx, self.python_interface)
         all_nodes = []
-        prefix = f"{ctx.compilation_state.prefix}-{self.short_name}-" if ctx.compilation_state is not None else ""
+        prefix = ctx.compilation_state.prefix if ctx.compilation_state is not None else ""
 
         with FlyteContextManager.with_context(
             ctx.with_compilation_state(CompilationState(prefix=prefix, task_resolver=self))
@@ -690,7 +712,8 @@ def workflow(
     _workflow_function=None,
     failure_policy: Optional[WorkflowFailurePolicy] = None,
     interruptible: bool = False,
-):
+    docs: Optional[Documentation] = None,
+) -> WorkflowBase:
     """
     This decorator declares a function to be a Flyte workflow. Workflows are declarative entities that construct a DAG
     of tasks using the data flow between tasks.
@@ -718,6 +741,7 @@ def workflow(
     :param _workflow_function: This argument is implicitly passed and represents the decorated function.
     :param failure_policy: Use the options in flytekit.WorkflowFailurePolicy
     :param interruptible: Whether or not tasks launched from this workflow are by default interruptible
+    :param docs: Description entity for the workflow
     """
 
     def wrapper(fn):
@@ -730,8 +754,10 @@ def workflow(
             metadata=workflow_metadata,
             default_metadata=workflow_metadata_defaults,
             docstring=Docstring(callable_=fn),
+            docs=docs,
         )
         workflow_instance.compile()
+        update_wrapper(workflow_instance, fn)
         return workflow_instance
 
     if _workflow_function:
@@ -771,7 +797,7 @@ def reference_workflow(
     """
 
     def wrapper(fn) -> ReferenceWorkflow:
-        interface = transform_signature_to_interface(inspect.signature(fn))
+        interface = transform_function_to_interface(fn)
         return ReferenceWorkflow(project, domain, name, version, interface.inputs, interface.outputs)
 
     return wrapper
