@@ -1,11 +1,13 @@
+import base64
 import os
 import pathlib
 
 import requests
 
-from flytekit.common.exceptions import user
 from flytekit.core.data_persistence import DataPersistence, DataPersistencePlugins
+from flytekit.exceptions import user
 from flytekit.loggers import logger
+from flytekit.tools import script_mode
 
 
 class HttpPersistence(DataPersistence):
@@ -54,7 +56,23 @@ class HttpPersistence(DataPersistence):
             writer.write(rsp.content)
 
     def put(self, from_path: str, to_path: str, recursive: bool = False):
-        raise user.FlyteAssertion("Writing data to HTTP endpoint is not currently supported.")
+        if recursive:
+            raise user.FlyteAssertion("Recursive writing data to HTTP endpoint is not currently supported.")
+
+        md5, _ = script_mode.hash_file(from_path)
+        encoded_md5 = base64.b64encode(md5)
+        with open(from_path, "+rb") as local_file:
+            content = local_file.read()
+            content_length = len(content)
+            rsp = requests.put(
+                to_path, data=content, headers={"Content-Length": str(content_length), "Content-MD5": encoded_md5}
+            )
+
+            if rsp.status_code != self._HTTP_OK:
+                raise user.FlyteValueException(
+                    rsp.status_code,
+                    f"Request to send data {to_path} failed.",
+                )
 
     def construct_path(self, add_protocol: bool, add_prefix: bool, *paths) -> str:
         raise user.FlyteAssertion(
