@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 from flyteidl.admin import task_execution_pb2 as _task_execution_pb2
 
 from flytekit.models import common as _common
@@ -9,7 +7,15 @@ from flytekit.models.core import identifier as _identifier
 
 class TaskExecutionClosure(_common.FlyteIdlEntity):
     def __init__(
-        self, phase, logs, started_at, duration, created_at, updated_at, output_uri=None, error=None,
+        self,
+        phase,
+        logs,
+        started_at,
+        duration,
+        created_at,
+        updated_at,
+        output_uri=None,
+        error=None,
     ):
         """
         :param int phase: Enum value from flytekit.models.core.execution.TaskExecutionPhase
