@@ -1,7 +1,6 @@
 import abc as _abc
 import json as _json
 
-import six as _six
 from flyteidl.admin import common_pb2 as _common_pb2
 from google.protobuf import json_format as _json_format
 from google.protobuf import struct_pb2 as _struct
@@ -58,7 +57,7 @@ class FlyteIdlEntity(object, metaclass=FlyteType):
         """
         :rtype: Text
         """
-        return _six.text_type(self.to_flyte_idl())
+        return str(self.to_flyte_idl())
 
     def verbose_string(self):
         """
@@ -66,6 +65,9 @@ class FlyteIdlEntity(object, metaclass=FlyteType):
         """
         return self.short_string()
 
+    def serialize_to_string(self) -> str:
+        return self.to_flyte_idl().SerializeToString()
+
     @property
     def is_empty(self):
         return len(self.to_flyte_idl().SerializeToString()) == 0
@@ -333,7 +335,7 @@ class Labels(FlyteIdlEntity):
         """
         :rtype: dict[Text, Text]
         """
-        return _common_pb2.Labels(values={k: v for k, v in _six.iteritems(self.values)})
+        return _common_pb2.Labels(values={k: v for k, v in self.values.items()})
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
@@ -341,7 +343,7 @@ class Labels(FlyteIdlEntity):
         :param flyteidl.admin.common_pb2.Labels pb2_object:
         :rtype: Labels
         """
-        return cls({k: v for k, v in _six.iteritems(pb2_object.values)})
+        return cls({k: v for k, v in pb2_object.values.items()})
 
 
 class Annotations(FlyteIdlEntity):
@@ -361,7 +363,7 @@ class Annotations(FlyteIdlEntity):
         """
         :rtype: _common_pb2.Annotations
         """
-        return _common_pb2.Annotations(values={k: v for k, v in _six.iteritems(self.values)})
+        return _common_pb2.Annotations(values={k: v for k, v in self.values.items()})
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
@@ -369,7 +371,7 @@ class Annotations(FlyteIdlEntity):
         :param flyteidl.admin.common_pb2.Annotations pb2_object:
         :rtype: Annotations
         """
-        return cls({k: v for k, v in _six.iteritems(pb2_object.values)})
+        return cls({k: v for k, v in pb2_object.values.items()})
 
 
 class UrlBlob(FlyteIdlEntity):
@@ -413,10 +415,11 @@ class UrlBlob(FlyteIdlEntity):
 class AuthRole(FlyteIdlEntity):
     def __init__(self, assumable_iam_role=None, kubernetes_service_account=None):
         """
-        At most one of assumable_iam_role or kubernetes_service_account can be set.
+        Either one or both of the assumable IAM role and/or the K8s service account can be set.
         :param Text assumable_iam_role: IAM identity with set permissions policies.
-        :param Text kubernetes_service_account: Provides an identity for workflow execution resources. Flyte deployment
-            administrators are responsible for handling permissions as they relate to the service account.
+        :param Text kubernetes_service_account: Provides an identity for workflow execution resources.
+          Flyte deployment administrators are responsible for handling permissions as they
+          relate to the service account.
         """
         self._assumable_iam_role = assumable_iam_role
         self._kubernetes_service_account = kubernetes_service_account
