@@ -1,12 +1,39 @@
+from __future__ import annotations
+
+import datetime
+import typing
+
+import flyteidl
 import flyteidl.admin.execution_pb2 as _execution_pb2
 import flyteidl.admin.node_execution_pb2 as _node_execution_pb2
 import flyteidl.admin.task_execution_pb2 as _task_execution_pb2
 import pytz as _pytz
 
+import flytekit
 from flytekit.models import common as _common_models
 from flytekit.models import literals as _literals_models
+from flytekit.models import security
 from flytekit.models.core import execution as _core_execution
 from flytekit.models.core import identifier as _identifier
+from flytekit.models.node_execution import DynamicWorkflowNodeMetadata
+
+
+class SystemMetadata(_common_models.FlyteIdlEntity):
+    def __init__(self, execution_cluster: str):
+        self._execution_cluster = execution_cluster
+
+    @property
+    def execution_cluster(self) -> str:
+        return self._execution_cluster
+
+    def to_flyte_idl(self) -> flyteidl.admin.execution_pb2.SystemMetadata:
+        return _execution_pb2.SystemMetadata(execution_cluster=self.execution_cluster)
+
+    @classmethod
+    def from_flyte_idl(cls, pb2_object: flyteidl.admin.execution_pb2.SystemMetadata) -> SystemMetadata:
+        return cls(
+            execution_cluster=pb2_object.execution_cluster,
+        )
 
 
 class ExecutionMetadata(_common_models.FlyteIdlEntity):
@@ -15,46 +42,102 @@ class ExecutionMetadata(_common_models.FlyteIdlEntity):
         SCHEDULED = 1
         SYSTEM = 2
 
-    def __init__(self, mode, principal, nesting):
+    def __init__(
+        self,
+        mode: int,
+        principal: str,
+        nesting: int,
+        scheduled_at: typing.Optional[datetime.datetime] = None,
+        parent_node_execution: typing.Optional[_identifier.NodeExecutionIdentifier] = None,
+        reference_execution: typing.Optional[_identifier.WorkflowExecutionIdentifier] = None,
+        system_metadata: typing.Optional[SystemMetadata] = None,
+    ):
         """
-        :param int mode: An enum value from ExecutionMetadata.ExecutionMode which specifies how the job started.
-        :param Text principal: The entity that triggered the execution
-        :param int nesting: An integer representing how deeply nested the workflow is (i.e. was it triggered by a parent
+        :param mode: An enum value from ExecutionMetadata.ExecutionMode which specifies how the job started.
+        :param principal: The entity that triggered the execution
+        :param nesting: An integer representing how deeply nested the workflow is (i.e. was it triggered by a parent
             workflow)
+        :param scheduled_at: For scheduled executions, the requested time for execution for this specific schedule invocation.
+        :param parent_node_execution: Which subworkflow node (if any) launched this execution
+        :param reference_execution: Optional, reference workflow execution related to this execution
+        :param system_metadata: Optional, platform-specific metadata about the execution.
         """
         self._mode = mode
         self._principal = principal
         self._nesting = nesting
+        self._scheduled_at = scheduled_at
+        self._parent_node_execution = parent_node_execution
+        self._reference_execution = reference_execution
+        self._system_metadata = system_metadata
 
     @property
-    def mode(self):
+    def mode(self) -> int:
         """
         An enum value from ExecutionMetadata.ExecutionMode which specifies how the job started.
-        :rtype: int
         """
         return self._mode
 
     @property
-    def principal(self):
+    def principal(self) -> str:
         """
         The entity that triggered the execution
-        :rtype: Text
         """
         return self._principal
 
     @property
-    def nesting(self):
+    def nesting(self) -> int:
         """
         An integer representing how deeply nested the workflow is (i.e. was it triggered by a parent workflow)
-        :rtype: int
         """
         return self._nesting
 
+    @property
+    def scheduled_at(self) -> datetime.datetime:
+        """
+        For scheduled executions, the requested time for execution for this specific schedule invocation.
+        """
+        return self._scheduled_at
+
+    @property
+    def parent_node_execution(self) -> _identifier.NodeExecutionIdentifier:
+        """
+        Which subworkflow node (if any) launched this execution
+        """
+        return self._parent_node_execution
+
+    @property
+    def reference_execution(self) -> _identifier.WorkflowExecutionIdentifier:
+        """
+        Optional, reference workflow execution related to this execution
+        """
+        return self._reference_execution
+
+    @property
+    def system_metadata(self) -> SystemMetadata:
+        """
+        Optional, platform-specific metadata about the execution.
+        """
+        return self._system_metadata
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.execution_pb2.ExecutionMetadata
         """
-        return _execution_pb2.ExecutionMetadata(mode=self.mode, principal=self.principal, nesting=self.nesting)
+        p = _execution_pb2.ExecutionMetadata(
+            mode=self.mode,
+            principal=self.principal,
+            nesting=self.nesting,
+            parent_node_execution=self.parent_node_execution.to_flyte_idl()
+            if self.parent_node_execution is not None
+            else None,
+            reference_execution=self.reference_execution.to_flyte_idl()
+            if self.reference_execution is not None
+            else None,
+            system_metadata=self.system_metadata.to_flyte_idl() if self.system_metadata is not None else None,
+        )
+        if self.scheduled_at is not None:
+            p.scheduled_at.FromDatetime(self.scheduled_at)
+        return p
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
@@ -66,6 +149,16 @@ class ExecutionMetadata(_common_models.FlyteIdlEntity):
             mode=pb2_object.mode,
             principal=pb2_object.principal,
             nesting=pb2_object.nesting,
+            scheduled_at=pb2_object.scheduled_at.ToDatetime() if pb2_object.HasField("scheduled_at") else None,
+            parent_node_execution=_identifier.NodeExecutionIdentifier.from_flyte_idl(pb2_object.parent_node_execution)
+            if pb2_object.HasField("parent_node_execution")
+            else None,
+            reference_execution=_identifier.WorkflowExecutionIdentifier.from_flyte_idl(pb2_object.reference_execution)
+            if pb2_object.HasField("reference_execution")
+            else None,
+            system_metadata=SystemMetadata.from_flyte_idl(pb2_object.system_metadata)
+            if pb2_object.HasField("system_metadata")
+            else None,
         )
 
 
@@ -79,7 +172,10 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
         labels=None,
         annotations=None,
         auth_role=None,
+        raw_output_data_config=None,
         max_parallelism=None,
+        security_context: typing.Optional[security.SecurityContext] = None,
+        overwrite_cache: bool = None,
     ):
         """
         :param flytekit.models.core.identifier.Identifier launch_plan: Launch plan unique identifier to execute
@@ -89,6 +185,7 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
         :param flytekit.models.common.Labels labels: Labels to apply to the execution.
         :param flytekit.models.common.Annotations annotations: Annotations to apply to the execution
         :param flytekit.models.common.AuthRole auth_role: The authorization method with which to execute the workflow.
+        :param raw_output_data_config: Optional location of offloaded data for things like S3, etc.
         :param max_parallelism int: Controls the maximum number of tasknodes that can be run in parallel for the entire
             workflow. This is useful to achieve fairness. Note: MapTasks are regarded as one unit, and
             parallelism/concurrency of MapTasks is independent from this.
@@ -101,7 +198,10 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
         self._labels = labels or _common_models.Labels({})
         self._annotations = annotations or _common_models.Annotations({})
         self._auth_role = auth_role or _common_models.AuthRole()
+        self._raw_output_data_config = raw_output_data_config
         self._max_parallelism = max_parallelism
+        self._security_context = security_context
+        self.overwrite_cache = overwrite_cache
 
     @property
     def launch_plan(self):
@@ -153,10 +253,21 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
         """
         return self._auth_role
 
+    @property
+    def raw_output_data_config(self):
+        """
+        :rtype: flytekit.models.common.RawOutputDataConfig
+        """
+        return self._raw_output_data_config
+
     @property
     def max_parallelism(self) -> int:
         return self._max_parallelism
 
+    @property
+    def security_context(self) -> typing.Optional[security.SecurityContext]:
+        return self._security_context
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.execution_pb2.ExecutionSpec
@@ -169,7 +280,12 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
             labels=self.labels.to_flyte_idl(),
             annotations=self.annotations.to_flyte_idl(),
             auth_role=self._auth_role.to_flyte_idl() if self.auth_role else None,
+            raw_output_data_config=self._raw_output_data_config.to_flyte_idl()
+            if self._raw_output_data_config
+            else None,
             max_parallelism=self.max_parallelism,
+            security_context=self.security_context.to_flyte_idl() if self.security_context else None,
+            overwrite_cache=self.overwrite_cache,
         )
 
     @classmethod
@@ -186,7 +302,14 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
             labels=_common_models.Labels.from_flyte_idl(p.labels),
             annotations=_common_models.Annotations.from_flyte_idl(p.annotations),
             auth_role=_common_models.AuthRole.from_flyte_idl(p.auth_role),
+            raw_output_data_config=_common_models.RawOutputDataConfig.from_flyte_idl(p.raw_output_data_config)
+            if p.HasField("raw_output_data_config")
+            else None,
             max_parallelism=p.max_parallelism,
+            security_context=security.SecurityContext.from_flyte_idl(p.security_context)
+            if p.security_context
+            else None,
+            overwrite_cache=p.overwrite_cache,
         )
 
 
@@ -238,7 +361,6 @@ class Execution(_common_models.FlyteIdlEntity):
     def __init__(self, id, spec, closure):
         """
         :param flytekit.models.core.identifier.WorkflowExecutionIdentifier id:
-        :param Text id:
         :param ExecutionSpec spec:
         :param ExecutionClosure closure:
         """
@@ -290,48 +412,94 @@ class Execution(_common_models.FlyteIdlEntity):
         )
 
 
+class AbortMetadata(_common_models.FlyteIdlEntity):
+    def __init__(self, cause: str, principal: str):
+        self._cause = cause
+        self._principal = principal
+
+    @property
+    def cause(self) -> str:
+        return self._cause
+
+    @property
+    def principal(self) -> str:
+        return self._principal
+
+    def to_flyte_idl(self) -> flyteidl.admin.execution_pb2.AbortMetadata:
+        return _execution_pb2.AbortMetadata(cause=self.cause, principal=self.principal)
+
+    @classmethod
+    def from_flyte_idl(cls, pb2_object: flyteidl.admin.execution_pb2.AbortMetadata) -> AbortMetadata:
+        return cls(
+            cause=pb2_object.cause,
+            principal=pb2_object.principal,
+        )
+
+
 class ExecutionClosure(_common_models.FlyteIdlEntity):
-    def __init__(self, phase, started_at, error=None, outputs=None):
+    def __init__(
+        self,
+        phase: int,
+        started_at: datetime.datetime,
+        duration: datetime.timedelta,
+        error: typing.Optional[flytekit.models.core.execution.ExecutionError] = None,
+        outputs: typing.Optional[LiteralMapBlob] = None,
+        abort_metadata: typing.Optional[AbortMetadata] = None,
+        created_at: typing.Optional[datetime.datetime] = None,
+        updated_at: typing.Optional[datetime.datetime] = None,
+    ):
         """
-        :param int phase: From the flytekit.models.core.execution.WorkflowExecutionPhase enum
-        :param datetime.datetime started_at:
-        :param flytekit.models.core.execution.ExecutionError error:
-        :param LiteralMapBlob outputs:
+        :param phase: From the flytekit.models.core.execution.WorkflowExecutionPhase enum
+        :param started_at:
+        :param duration: Duration for which the execution has been running.
+        :param error:
+        :param outputs:
+        :param abort_metadata: Specifies metadata around an aborted workflow execution.
         """
         self._phase = phase
         self._started_at = started_at
+        self._duration = duration
         self._error = error
         self._outputs = outputs
+        self._abort_metadata = abort_metadata
+        self._created_at = created_at
+        self._updated_at = updated_at
 
     @property
-    def error(self):
-        """
-        :rtype: flytekit.models.core.execution.ExecutionError
-        """
+    def error(self) -> flytekit.models.core.execution.ExecutionError:
         return self._error
 
     @property
-    def phase(self):
+    def phase(self) -> int:
         """
         From the flytekit.models.core.execution.WorkflowExecutionPhase enum
-        :rtype: int
         """
         return self._phase
 
     @property
-    def started_at(self):
-        """
-        :rtype: datetime.datetime
-        """
+    def started_at(self) -> datetime.datetime:
         return self._started_at
 
     @property
-    def outputs(self):
-        """
-        :rtype: LiteralMapBlob
-        """
+    def duration(self) -> datetime.timedelta:
+        return self._duration
+
+    @property
+    def created_at(self) -> typing.Optional[datetime.datetime]:
+        return self._created_at
+
+    @property
+    def updated_at(self) -> typing.Optional[datetime.datetime]:
+        return self._updated_at
+
+    @property
+    def outputs(self) -> LiteralMapBlob:
         return self._outputs
 
+    @property
+    def abort_metadata(self) -> AbortMetadata:
+        return self._abort_metadata
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.execution_pb2.ExecutionClosure
@@ -340,8 +508,14 @@ class ExecutionClosure(_common_models.FlyteIdlEntity):
             phase=self.phase,
             error=self.error.to_flyte_idl() if self.error is not None else None,
             outputs=self.outputs.to_flyte_idl() if self.outputs is not None else None,
+            abort_metadata=self.abort_metadata.to_flyte_idl() if self.abort_metadata is not None else None,
         )
         obj.started_at.FromDatetime(self.started_at.astimezone(_pytz.UTC).replace(tzinfo=None))
+        obj.duration.FromTimedelta(self.duration)
+        if self.created_at:
+            obj.created_at.FromDatetime(self.created_at.astimezone(_pytz.UTC).replace(tzinfo=None))
+        if self.updated_at:
+            obj.updated_at.FromDatetime(self.updated_at.astimezone(_pytz.UTC).replace(tzinfo=None))
         return obj
 
     @classmethod
@@ -356,11 +530,22 @@ class ExecutionClosure(_common_models.FlyteIdlEntity):
         outputs = None
         if pb2_object.HasField("outputs"):
             outputs = LiteralMapBlob.from_flyte_idl(pb2_object.outputs)
+        abort_metadata = None
+        if pb2_object.HasField("abort_metadata"):
+            abort_metadata = AbortMetadata.from_flyte_idl(pb2_object.abort_metadata)
         return cls(
             error=error,
             outputs=outputs,
             phase=pb2_object.phase,
             started_at=pb2_object.started_at.ToDatetime().replace(tzinfo=_pytz.UTC),
+            duration=pb2_object.duration.ToTimedelta(),
+            abort_metadata=abort_metadata,
+            created_at=pb2_object.created_at.ToDatetime().replace(tzinfo=_pytz.UTC)
+            if pb2_object.HasField("created_at")
+            else None,
+            updated_at=pb2_object.updated_at.ToDatetime().replace(tzinfo=_pytz.UTC)
+            if pb2_object.HasField("updated_at")
+            else None,
         )
 
 
@@ -403,8 +588,8 @@ class _CommonDataResponse(_common_models.FlyteIdlEntity):
         """
         :param _common_models.UrlBlob inputs:
         :param _common_models.UrlBlob outputs:
-        :param _literals_pb2.LiteralMap full_inputs:
-        :param _literals_pb2.LiteralMap full_outputs:
+        :param _literals_models.LiteralMap full_inputs:
+        :param _literals_models.LiteralMap full_outputs:
         """
         self._inputs = inputs
         self._outputs = outputs
@@ -428,14 +613,14 @@ class _CommonDataResponse(_common_models.FlyteIdlEntity):
     @property
     def full_inputs(self):
         """
-        :rtype: _literals_pb2.LiteralMap
+        :rtype: _literals_models.LiteralMap
         """
         return self._full_inputs
 
     @property
     def full_outputs(self):
         """
-        :rtype: _literals_pb2.LiteralMap
+        :rtype: _literals_models.LiteralMap
         """
         return self._full_outputs
 
@@ -493,6 +678,14 @@ class TaskExecutionGetDataResponse(_CommonDataResponse):
 
 
 class NodeExecutionGetDataResponse(_CommonDataResponse):
+    def __init__(self, *args, dynamic_workflow: typing.Optional[DynamicWorkflowNodeMetadata] = None, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._dynamic_workflow = dynamic_workflow
+
+    @property
+    def dynamic_workflow(self) -> typing.Optional[DynamicWorkflowNodeMetadata]:
+        return self._dynamic_workflow
+
     @classmethod
     def from_flyte_idl(cls, pb2_object):
         """
@@ -504,6 +697,9 @@ class NodeExecutionGetDataResponse(_CommonDataResponse):
             outputs=_common_models.UrlBlob.from_flyte_idl(pb2_object.outputs),
             full_inputs=_literals_models.LiteralMap.from_flyte_idl(pb2_object.full_inputs),
             full_outputs=_literals_models.LiteralMap.from_flyte_idl(pb2_object.full_outputs),
+            dynamic_workflow=DynamicWorkflowNodeMetadata.from_flyte_idl(pb2_object.dynamic_workflow)
+            if pb2_object.HasField("dynamic_workflow")
+            else None,
         )
 
     def to_flyte_idl(self):
@@ -515,4 +711,5 @@ class NodeExecutionGetDataResponse(_CommonDataResponse):
             outputs=self.outputs.to_flyte_idl(),
             full_inputs=self.full_inputs.to_flyte_idl(),
             full_outputs=self.full_outputs.to_flyte_idl(),
+            dynamic_workflow=self.dynamic_workflow.to_flyte_idl() if self.dynamic_workflow else None,
         )
