@@ -1,25 +1,19 @@
 import json as _json
 import typing
 
-import six as _six
 from flyteidl.admin import task_pb2 as _admin_task
 from flyteidl.core import compiler_pb2 as _compiler
 from flyteidl.core import literals_pb2 as _literals_pb2
 from flyteidl.core import tasks_pb2 as _core_task
-from flyteidl.plugins import pytorch_pb2 as _pytorch_task
-from flyteidl.plugins import spark_pb2 as _spark_task
-from flyteidl.plugins import tensorflow_pb2 as _tensorflow_task
 from google.protobuf import json_format as _json_format
 from google.protobuf import struct_pb2 as _struct
 
-from flytekit.common.exceptions import user as _user_exceptions
 from flytekit.models import common as _common
 from flytekit.models import interface as _interface
 from flytekit.models import literals as _literals
 from flytekit.models import security as _sec
 from flytekit.models.core import identifier as _identifier
-from flytekit.plugins import flyteidl as _lazy_flyteidl
-from flytekit.sdk.spark_types import SparkType as _spark_type
+from flytekit.models.documentation import Documentation
 
 
 class Resources(_common.FlyteIdlEntity):
@@ -29,6 +23,7 @@ class Resources(_common.FlyteIdlEntity):
         GPU = _core_task.Resources.GPU
         MEMORY = _core_task.Resources.MEMORY
         STORAGE = _core_task.Resources.STORAGE
+        EPHEMERAL_STORAGE = _core_task.Resources.EPHEMERAL_STORAGE
 
     class ResourceEntry(_common.FlyteIdlEntity):
         def __init__(self, name, value):
@@ -181,6 +176,8 @@ class TaskMetadata(_common.FlyteIdlEntity):
         interruptible,
         discovery_version,
         deprecated_error_message,
+        cache_serializable,
+        pod_template_name,
     ):
         """
         Information needed at runtime to determine behavior such as whether or not outputs are discoverable, timeouts,
@@ -198,6 +195,9 @@ class TaskMetadata(_common.FlyteIdlEntity):
             task are the same and the discovery_version is also the same.
         :param Text deprecated: This string can be used to mark the task as deprecated.  Consumers of the task will
             receive deprecation warnings.
+        :param bool cache_serializable: Whether or not caching operations are executed in serial. This means only a
+            single instance over identical inputs is executed, other concurrent executions wait for the cached results.
+        :param pod_template_name: The name of the existing PodTemplate resource which will be used in this task.
         """
         self._discoverable = discoverable
         self._runtime = runtime
@@ -206,6 +206,8 @@ class TaskMetadata(_common.FlyteIdlEntity):
         self._retries = retries
         self._discovery_version = discovery_version
         self._deprecated_error_message = deprecated_error_message
+        self._cache_serializable = cache_serializable
+        self._pod_template_name = pod_template_name
 
     @property
     def discoverable(self):
@@ -266,6 +268,23 @@ class TaskMetadata(_common.FlyteIdlEntity):
         """
         return self._deprecated_error_message
 
+    @property
+    def cache_serializable(self):
+        """
+        Whether or not caching operations are executed in serial. This means only a single instance over identical
+        inputs is executed, other concurrent executions wait for the cached results.
+        :rtype: bool
+        """
+        return self._cache_serializable
+
+    @property
+    def pod_template_name(self):
+        """
+        The name of the existing PodTemplate resource which will be used in this task.
+        :rtype: Text
+        """
+        return self._pod_template_name
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.task_pb2.TaskMetadata
@@ -277,6 +296,8 @@ class TaskMetadata(_common.FlyteIdlEntity):
             interruptible=self.interruptible,
             discovery_version=self.discovery_version,
             deprecated_error_message=self.deprecated_error_message,
+            cache_serializable=self.cache_serializable,
+            pod_template_name=self.pod_template_name,
         )
         if self.timeout:
             tm.timeout.FromTimedelta(self.timeout)
@@ -296,6 +317,8 @@ class TaskMetadata(_common.FlyteIdlEntity):
             retries=_literals.RetryStrategy.from_flyte_idl(pb2_object.retries),
             discovery_version=pb2_object.discovery_version,
             deprecated_error_message=pb2_object.deprecated_error_message,
+            cache_serializable=pb2_object.cache_serializable,
+            pod_template_name=pb2_object.pod_template_name,
         )
 
 
@@ -312,6 +335,7 @@ class TaskTemplate(_common.FlyteIdlEntity):
         security_context=None,
         config=None,
         k8s_pod=None,
+        sql=None,
     ):
         """
         A task template represents the full set of information necessary to perform a unit of work in the Flyte system.
@@ -334,9 +358,14 @@ class TaskTemplate(_common.FlyteIdlEntity):
         :param dict[str, str] config: For plugin tasks this represents additional configuration information to be used
             in tandem with the custom.
         :param K8sPod k8s_pod: Alternative to the container used to execute this task.
+        :param Sql sql: This is used to execute query in FlytePropeller instead of running container or k8s_pod.
         """
-        if container is not None and k8s_pod is not None:
-            raise ValueError("At most one of container or k8s_pod can be set")
+        if (
+            (container is not None and k8s_pod is not None)
+            or (container is not None and sql is not None)
+            or (k8s_pod is not None and sql is not None)
+        ):
+            raise ValueError("At most one of container, k8s_pod or sql can be set")
         self._id = id
         self._type = type
         self._metadata = metadata
@@ -347,6 +376,7 @@ class TaskTemplate(_common.FlyteIdlEntity):
         self._config = config
         self._security_context = security_context
         self._k8s_pod = k8s_pod
+        self._sql = sql
 
     @property
     def id(self):
@@ -417,6 +447,10 @@ class TaskTemplate(_common.FlyteIdlEntity):
     def k8s_pod(self):
         return self._k8s_pod
 
+    @property
+    def sql(self):
+        return self._sql
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.core.tasks_pb2.TaskTemplate
@@ -432,6 +466,7 @@ class TaskTemplate(_common.FlyteIdlEntity):
             security_context=self.security_context.to_flyte_idl() if self.security_context else None,
             config={k: v for k, v in self.config.items()} if self.config is not None else None,
             k8s_pod=self.k8s_pod.to_flyte_idl() if self.k8s_pod else None,
+            sql=self.sql.to_flyte_idl() if self.sql else None,
         )
         return task_template
 
@@ -450,19 +485,22 @@ class TaskTemplate(_common.FlyteIdlEntity):
             container=Container.from_flyte_idl(pb2_object.container) if pb2_object.HasField("container") else None,
             task_type_version=pb2_object.task_type_version,
             security_context=_sec.SecurityContext.from_flyte_idl(pb2_object.security_context)
-            if pb2_object.security_context
+            if pb2_object.security_context and pb2_object.security_context.ByteSize() > 0
             else None,
             config={k: v for k, v in pb2_object.config.items()} if pb2_object.config is not None else None,
             k8s_pod=K8sPod.from_flyte_idl(pb2_object.k8s_pod) if pb2_object.HasField("k8s_pod") else None,
+            sql=Sql.from_flyte_idl(pb2_object.sql) if pb2_object.HasField("sql") else None,
         )
 
 
 class TaskSpec(_common.FlyteIdlEntity):
-    def __init__(self, template):
+    def __init__(self, template: TaskTemplate, docs: typing.Optional[Documentation] = None):
         """
         :param TaskTemplate template:
+        :param Documentation docs:
         """
         self._template = template
+        self._docs = docs
 
     @property
     def template(self):
@@ -471,11 +509,20 @@ class TaskSpec(_common.FlyteIdlEntity):
         """
         return self._template
 
+    @property
+    def docs(self):
+        """
+        :rtype: Description entity for the task
+        """
+        return self._docs
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.tasks_pb2.TaskSpec
         """
-        return _admin_task.TaskSpec(template=self.template.to_flyte_idl())
+        return _admin_task.TaskSpec(
+            template=self.template.to_flyte_idl(), description=self.docs.to_flyte_idl() if self.docs else None
+        )
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
@@ -483,7 +530,10 @@ class TaskSpec(_common.FlyteIdlEntity):
         :param flyteidl.admin.tasks_pb2.TaskSpec pb2_object:
         :rtype: TaskSpec
         """
-        return cls(TaskTemplate.from_flyte_idl(pb2_object.template))
+        return cls(
+            TaskTemplate.from_flyte_idl(pb2_object.template),
+            Documentation.from_flyte_idl(pb2_object.description) if pb2_object.description else None,
+        )
 
 
 class Task(_common.FlyteIdlEntity):
@@ -590,146 +640,6 @@ class CompiledTask(_common.FlyteIdlEntity):
         return cls(template=TaskTemplate.from_flyte_idl(pb2_object.template))
 
 
-class SparkJob(_common.FlyteIdlEntity):
-    def __init__(
-        self,
-        spark_type,
-        application_file,
-        main_class,
-        spark_conf,
-        hadoop_conf,
-        executor_path,
-    ):
-        """
-        This defines a SparkJob target.  It will execute the appropriate SparkJob.
-
-        :param application_file: The main application file to execute.
-        :param dict[Text, Text] spark_conf: A definition of key-value pairs for spark config for the job.
-        :param dict[Text, Text] hadoop_conf: A definition of key-value pairs for hadoop config for the job.
-        """
-        self._application_file = application_file
-        self._spark_type = spark_type
-        self._main_class = main_class
-        self._executor_path = executor_path
-        self._spark_conf = spark_conf
-        self._hadoop_conf = hadoop_conf
-
-    def with_overrides(
-        self, new_spark_conf: typing.Dict[str, str] = None, new_hadoop_conf: typing.Dict[str, str] = None
-    ) -> "SparkJob":
-        if not new_spark_conf:
-            new_spark_conf = self.spark_conf
-
-        if not new_hadoop_conf:
-            new_hadoop_conf = self.hadoop_conf
-
-        return SparkJob(
-            spark_type=self.spark_type,
-            application_file=self.application_file,
-            main_class=self.main_class,
-            spark_conf=new_spark_conf,
-            hadoop_conf=new_hadoop_conf,
-            executor_path=self.executor_path,
-        )
-
-    @property
-    def main_class(self):
-        """
-        The main class to execute
-        :rtype: Text
-        """
-        return self._main_class
-
-    @property
-    def spark_type(self):
-        """
-        Spark Job Type
-        :rtype: Text
-        """
-        return self._spark_type
-
-    @property
-    def application_file(self):
-        """
-        The main application file to execute
-        :rtype: Text
-        """
-        return self._application_file
-
-    @property
-    def executor_path(self):
-        """
-        The python executable to use
-        :rtype: Text
-        """
-        return self._executor_path
-
-    @property
-    def spark_conf(self):
-        """
-        A definition of key-value pairs for spark config for the job.
-         :rtype: dict[Text, Text]
-        """
-        return self._spark_conf
-
-    @property
-    def hadoop_conf(self):
-        """
-         A definition of key-value pairs for hadoop config for the job.
-        :rtype: dict[Text, Text]
-        """
-        return self._hadoop_conf
-
-    def to_flyte_idl(self):
-        """
-        :rtype: flyteidl.plugins.spark_pb2.SparkJob
-        """
-
-        if self.spark_type == _spark_type.PYTHON:
-            application_type = _spark_task.SparkApplication.PYTHON
-        elif self.spark_type == _spark_type.JAVA:
-            application_type = _spark_task.SparkApplication.JAVA
-        elif self.spark_type == _spark_type.SCALA:
-            application_type = _spark_task.SparkApplication.SCALA
-        elif self.spark_type == _spark_type.R:
-            application_type = _spark_task.SparkApplication.R
-        else:
-            raise _user_exceptions.FlyteValidationException("Invalid Spark Application Type Specified")
-
-        return _spark_task.SparkJob(
-            applicationType=application_type,
-            mainApplicationFile=self.application_file,
-            mainClass=self.main_class,
-            executorPath=self.executor_path,
-            sparkConf=self.spark_conf,
-            hadoopConf=self.hadoop_conf,
-        )
-
-    @classmethod
-    def from_flyte_idl(cls, pb2_object):
-        """
-        :param flyteidl.plugins.spark_pb2.SparkJob pb2_object:
-        :rtype: SparkJob
-        """
-
-        application_type = _spark_type.PYTHON
-        if pb2_object.type == _spark_task.SparkApplication.JAVA:
-            application_type = _spark_type.JAVA
-        elif pb2_object.type == _spark_task.SparkApplication.SCALA:
-            application_type = _spark_type.SCALA
-        elif pb2_object.type == _spark_task.SparkApplication.R:
-            application_type = _spark_type.R
-
-        return cls(
-            type=application_type,
-            spark_conf=pb2_object.sparkConf,
-            application_file=pb2_object.mainApplicationFile,
-            main_class=pb2_object.mainClass,
-            hadoop_conf=pb2_object.hadoopConf,
-            executor_path=pb2_object.executorPath,
-        )
-
-
 class IOStrategy(_common.FlyteIdlEntity):
     """
     Provides methods to manage data in and out of the Raw container using Download Modes. This can only be used if DataLoadingConfig is enabled.
@@ -873,6 +783,9 @@ class Container(_common.FlyteIdlEntity):
         """
         return self._env
 
+    def add_env(self, key: str, val: str):
+        self._env[key] = val
+
     @property
     def config(self):
         """
@@ -898,8 +811,8 @@ class Container(_common.FlyteIdlEntity):
             command=self.command,
             args=self.args,
             resources=self.resources.to_flyte_idl(),
-            env=[_literals_pb2.KeyValuePair(key=k, value=v) for k, v in _six.iteritems(self.env)],
-            config=[_literals_pb2.KeyValuePair(key=k, value=v) for k, v in _six.iteritems(self.config)],
+            env=[_literals_pb2.KeyValuePair(key=k, value=v) for k, v in self.env.items()],
+            config=[_literals_pb2.KeyValuePair(key=k, value=v) for k, v in self.config.items()],
             data_config=self._data_loading_config.to_flyte_idl() if self._data_loading_config else None,
         )
 
@@ -984,122 +897,32 @@ class K8sPod(_common.FlyteIdlEntity):
         )
 
 
-class SidecarJob(_common.FlyteIdlEntity):
-    def __init__(self, pod_spec, primary_container_name, annotations=None, labels=None):
-        """
-        A sidecar job represents the full kubernetes pod spec and related metadata required for executing a sidecar
-        task.
+class Sql(_common.FlyteIdlEntity):
+    class Dialect(object):
+        ANSI = 0
+        HIVE = 1
 
-        :param pod_spec: k8s.io.api.core.v1.PodSpec
-        :param primary_container_name: Text
-        :param dict[Text, Text] annotations:
-        :param dict[Text, Text] labels:
+    def __init__(self, statement: str = None, dialect: int = 0):
         """
-        self._pod_spec = pod_spec
-        self._primary_container_name = primary_container_name
-        self._annotations = annotations
-        self._labels = labels
+        This defines a kubernetes pod target. It will build the pod target during task execution
+        """
+        self._statement = statement
+        self._dialect = dialect
 
     @property
-    def pod_spec(self):
-        """
-        :rtype: k8s.io.api.core.v1.PodSpec
-        """
-        return self._pod_spec
+    def statement(self) -> str:
+        return self._statement
 
     @property
-    def primary_container_name(self):
-        """
-        :rtype: Text
-        """
-        return self._primary_container_name
+    def dialect(self) -> int:
+        return self._dialect
 
-    @property
-    def annotations(self):
-        """
-        :rtype: dict[Text,Text]
-        """
-        return self._annotations
-
-    @property
-    def labels(self):
-        """
-        :rtype: dict[Text,Text]
-        """
-        return self._labels
-
-    def to_flyte_idl(self):
-        """
-        :rtype: flyteidl.core.tasks_pb2.SidecarJob
-        """
-        return _lazy_flyteidl.plugins.sidecar_pb2.SidecarJob(
-            pod_spec=self.pod_spec,
-            primary_container_name=self.primary_container_name,
-            annotations=self.annotations,
-            labels=self.labels,
-        )
+    def to_flyte_idl(self) -> _core_task.Sql:
+        return _core_task.Sql(statement=self.statement, dialect=self.dialect)
 
     @classmethod
-    def from_flyte_idl(cls, pb2_object):
-        """
-        :param flyteidl.admin.task_pb2.Task pb2_object:
-        :rtype: Container
-        """
+    def from_flyte_idl(cls, pb2_object: _core_task.Sql):
         return cls(
-            pod_spec=pb2_object.pod_spec,
-            primary_container_name=pb2_object.primary_container_name,
-            annotations=pb2_object.annotations,
-            labels=pb2_object.labels,
-        )
-
-
-class PyTorchJob(_common.FlyteIdlEntity):
-    def __init__(self, workers_count):
-        self._workers_count = workers_count
-
-    @property
-    def workers_count(self):
-        return self._workers_count
-
-    def to_flyte_idl(self):
-        return _pytorch_task.DistributedPyTorchTrainingTask(
-            workers=self.workers_count,
-        )
-
-    @classmethod
-    def from_flyte_idl(cls, pb2_object):
-        return cls(
-            workers_count=pb2_object.workers,
-        )
-
-
-class TensorFlowJob(_common.FlyteIdlEntity):
-    def __init__(self, workers_count, ps_replicas_count, chief_replicas_count):
-        self._workers_count = workers_count
-        self._ps_replicas_count = ps_replicas_count
-        self._chief_replicas_count = chief_replicas_count
-
-    @property
-    def workers_count(self):
-        return self._workers_count
-
-    @property
-    def ps_replicas_count(self):
-        return self._ps_replicas_count
-
-    @property
-    def chief_replicas_count(self):
-        return self._chief_replicas_count
-
-    def to_flyte_idl(self):
-        return _tensorflow_task.DistributedTensorflowTrainingTask(
-            workers=self.workers_count, ps_replicas=self.ps_replicas_count, chief_replicas=self.chief_replicas_count
-        )
-
-    @classmethod
-    def from_flyte_idl(cls, pb2_object):
-        return cls(
-            workers_count=pb2_object.workers,
-            ps_replicas_count=pb2_object.ps_replicas,
-            chief_replicas_count=pb2_object.chief_replicas,
+            statement=pb2_object.statement,
+            dialect=pb2_object.dialect,
         )
