@@ -3,23 +3,32 @@ from __future__ import annotations
 import os
 import pathlib
 import typing
+from dataclasses import dataclass, field
 from pathlib import Path
 
+from dataclasses_json import config, dataclass_json
+from marshmallow import fields
+
 from flytekit.core.context_manager import FlyteContext
 from flytekit.core.type_engine import TypeEngine, TypeTransformer
 from flytekit.models import types as _type_models
 from flytekit.models.core import types as _core_types
 from flytekit.models.literals import Blob, BlobMetadata, Literal, Scalar
 from flytekit.models.types import LiteralType
+from flytekit.types.file import FileExt
 
 T = typing.TypeVar("T")
+PathType = typing.Union[str, os.PathLike]
 
 
 def noop():
     ...
 
 
+@dataclass_json
+@dataclass
 class FlyteDirectory(os.PathLike, typing.Generic[T]):
+    path: PathType = field(default=None, metadata=config(mm_field=fields.String()))  # type: ignore
     """
     .. warning::
 
@@ -106,15 +115,16 @@ class FlyteDirectory(os.PathLike, typing.Generic[T]):
     field in the ``BlobType``.
     """
 
-    def __init__(self, path: str, downloader: typing.Callable = None, remote_directory=None):
+    def __init__(self, path: typing.Union[str, os.PathLike], downloader: typing.Callable = None, remote_directory=None):
         """
         :param path: The source path that users are expected to call open() on
         :param downloader: Optional function that can be passed that used to delay downloading of the actual fil
             until a user actually calls open().
         :param remote_directory: If the user wants to return something and also specify where it should be uploaded to.
         """
-
-        self._path = path
+        # Make this field public, so that the dataclass transformer can set a value for it
+        # https://github.com/flyteorg/flytekit/blob/bcc8541bd6227b532f8462563fe8aac902242b21/flytekit/core/type_engine.py#L298
+        self.path = path
         self._downloader = downloader or noop
         self._downloaded = False
         self._remote_directory = remote_directory
@@ -127,18 +137,20 @@ class FlyteDirectory(os.PathLike, typing.Generic[T]):
         if not self._downloaded:
             self._downloader()
             self._downloaded = True
-        return self._path
+        return self.path
 
     @classmethod
     def extension(cls) -> str:
         return ""
 
-    def __class_getitem__(cls, item: typing.Type) -> typing.Type[FlyteDirectory]:
+    def __class_getitem__(cls, item: typing.Union[typing.Type, str]) -> typing.Type[FlyteDirectory]:
         if item is None:
             return cls
-        item = str(item)
-        item = item.strip().lstrip("~").lstrip(".")
-        if item == "":
+
+        item_string = FileExt.check_and_convert_to_str(item)
+
+        item_string = item_string.strip().lstrip("~").lstrip(".")
+        if item_string == "":
             return cls
 
         class _SpecificFormatDirectoryClass(FlyteDirectory):
@@ -147,7 +159,7 @@ class FlyteDirectory(os.PathLike, typing.Generic[T]):
 
             @classmethod
             def extension(cls) -> str:
-                return item
+                return item_string
 
         return _SpecificFormatDirectoryClass
 
@@ -159,39 +171,34 @@ class FlyteDirectory(os.PathLike, typing.Generic[T]):
     def remote_directory(self) -> typing.Optional[str]:
         return self._remote_directory
 
-    @property
-    def path(self) -> str:
-        return self._path
-
     @property
     def remote_source(self) -> str:
         """
         If this is an input to a task, and the original path is s3://something, flytekit will download the
         directory for the user. In case the user wants access to the original path, it will be here.
         """
-        return self._remote_source
+        return typing.cast(str, self._remote_source)
 
     def download(self) -> str:
-        if self._downloaded:
-            return self.path
-        if self._downloader is not noop:
-            self._downloader()
-            self._downloaded = True
-            return self.path
-        else:
-            raise ValueError(f"Attempting to trigger download on non-downloadable folder {self}")
+        return self.__fspath__()
 
     def __repr__(self):
-        return self._path
+        return self.path
 
     def __str__(self):
-        return self._path
+        return self.path
 
 
 class FlyteDirToMultipartBlobTransformer(TypeTransformer[FlyteDirectory]):
     """
     This transformer handles conversion between the Python native FlyteDirectory class defined above, and the Flyte
     IDL literal/type of Multipart Blob. Please see the FlyteDirectory comments for additional information.
+
+    .. caution:
+
+       The transformer will not check if the given path is actually a directory. This is because the path could be
+       a remote reference.
+
     """
 
     def __init__(self):
@@ -205,6 +212,17 @@ class FlyteDirToMultipartBlobTransformer(TypeTransformer[FlyteDirectory]):
     def _blob_type(format: str) -> _core_types.BlobType:
         return _core_types.BlobType(format=format, dimensionality=_core_types.BlobType.BlobDimensionality.MULTIPART)
 
+    def assert_type(self, t: typing.Type[FlyteDirectory], v: typing.Union[FlyteDirectory, os.PathLike, str]):
+        if isinstance(v, FlyteDirectory) or isinstance(v, str) or isinstance(v, os.PathLike):
+            """
+            NOTE: we do not do a isdir check because the given path could be remote reference
+            """
+            return
+        raise TypeError(
+            f"No automatic conversion from {type(v)} declared type {t} to FlyteDirectory found."
+            f" Use (FlyteDirectory, str, os.PathLike)"
+        )
+
     def get_literal_type(self, t: typing.Type[FlyteDirectory]) -> LiteralType:
         return _type_models.LiteralType(blob=self._blob_type(format=FlyteDirToMultipartBlobTransformer.get_format(t)))
 
@@ -228,7 +246,7 @@ class FlyteDirToMultipartBlobTransformer(TypeTransformer[FlyteDirectory]):
                 return Literal(scalar=Scalar(blob=Blob(metadata=meta, uri=python_val._remote_source)))
 
             source_path = python_val.path
-            # If the user specified the remote_path to be False, that means no matter what, do not upload. Also if the
+            # If the user specified the remote_directory to be False, that means no matter what, do not upload. Also if the
             # path given is already a remote path, say https://www.google.com, the concept of uploading to the Flyte
             # blob store doesn't make sense.
             if python_val.remote_directory is False or ctx.file_access.is_remote(source_path):
@@ -280,17 +298,17 @@ class FlyteDirToMultipartBlobTransformer(TypeTransformer[FlyteDirectory]):
 
         expected_format = self.get_format(expected_python_type)
 
-        fd = FlyteDirectory[expected_format](local_folder, _downloader)
+        fd = FlyteDirectory.__class_getitem__(expected_format)(local_folder, _downloader)
         fd._remote_source = uri
 
         return fd
 
-    def guess_python_type(self, literal_type: LiteralType) -> typing.Type[T]:
+    def guess_python_type(self, literal_type: LiteralType) -> typing.Type[FlyteDirectory[typing.Any]]:
         if (
             literal_type.blob is not None
             and literal_type.blob.dimensionality == _core_types.BlobType.BlobDimensionality.MULTIPART
         ):
-            return FlyteDirectory[literal_type.blob.format]
+            return FlyteDirectory.__class_getitem__(literal_type.blob.format)
         raise ValueError(f"Transformer {self} cannot reverse {literal_type}")
 
 
