@@ -3,14 +3,18 @@ from __future__ import annotations
 import os
 import pathlib
 import typing
+from dataclasses import dataclass, field
+
+from dataclasses_json import config, dataclass_json
+from marshmallow import fields
 
 from flytekit.core.context_manager import FlyteContext
-from flytekit.core.type_engine import TypeEngine, TypeTransformer
+from flytekit.core.type_engine import TypeEngine, TypeTransformer, TypeTransformerFailedError
 from flytekit.loggers import logger
-from flytekit.models import types as _type_models
-from flytekit.models.core import types as _core_types
+from flytekit.models.core.types import BlobType
 from flytekit.models.literals import Blob, BlobMetadata, Literal, Scalar
 from flytekit.models.types import LiteralType
+from flytekit.types.pickle.pickle import FlytePickleTransformer
 
 
 def noop():
@@ -20,35 +24,41 @@ def noop():
 T = typing.TypeVar("T")
 
 
+@dataclass_json
+@dataclass
 class FlyteFile(os.PathLike, typing.Generic[T]):
+    path: typing.Union[str, os.PathLike] = field(default=None, metadata=config(mm_field=fields.String()))  # type: ignore
     """
     Since there is no native Python implementation of files and directories for the Flyte Blob type, (like how int
     exists for Flyte's Integer type) we need to create one so that users can express that their tasks take
-    in or return a file. There is ``pathlib.Path`` of course, (which is usable in flytekit as a return value, though
+    in or return a file. There is ``pathlib.Path`` of course, (which is usable in Flytekit as a return value, though
     not a return type), but it made more sense to create a new type esp. since we can add on additional properties.
 
-    Files (and directories) differ from the primitive types like floats and string in that flytekit typically uploads
+    Files (and directories) differ from the primitive types like floats and string in that Flytekit typically uploads
     the contents of the files to the blob store connected with your Flyte installation. That is, the Python native
     literal that represents a file is typically just the path to the file on the local filesystem. However in Flyte,
     an instance of a file is represented by a :py:class:`Blob <flytekit.models.literals.Blob>` literal,
-    with the ``uri`` field set to the location in the Flyte blob store (AWS/GCS etc.).
+    with the ``uri`` field set to the location in the Flyte blob store (AWS/GCS etc.). Take a look at the
+    :std:ref:`data handling doc <flyte:divedeep-data-management>` for a deeper discussion.
 
     We decided to not support ``pathlib.Path`` as an input/output type because if you wanted the automatic
     upload/download behavior, you should just use the ``FlyteFile`` type. If you do not, then a ``str`` works just as
     well.
 
     The prefix for where uploads go is set by the raw output data prefix setting, which should be set at registration
-    time. See the flytectl option for more information.
+    time in the launch plan. See the option listed under ``flytectl register examples --help`` for more information.
+    If not set in the launch plan, then your Flyte backend will specify a default. This default is itself configurable
+    as well. Contact your Flyte platform administrators to change or ascertain the value.
 
     In short, if a task returns ``"/path/to/file"`` and the task's signature is set to return ``FlyteFile``, then the
     contents of ``/path/to/file`` are uploaded.
 
-    You can also make it so that the upload does not happen. There are a few different types you use for
+    You can also make it so that the upload does not happen. There are different types of
     task/workflow signatures. Keep in mind that in the backend, in Admin and in the blob store, there is only one type
     that represents files, the :py:class:`Blob <flytekit.models.core.types.BlobType>` type.
 
-    Whether or not the uploading happens, and the behavior of the translation between Python native values and Flyte
-    literal values depends on a few things:
+    Whether the uploading happens or not, the behavior of the translation between Python native values and Flyte
+    literal values depends on a few attributes:
 
     * The declared Python type in the signature. These can be
       * :class:`python:flytekit.FlyteFile`
@@ -58,9 +68,9 @@ class FlyteFile(os.PathLike, typing.Generic[T]):
       * :py:class:`flytekit.FlyteFile`
       * :py:class:`pathlib.Path`
       * :py:class:`str`
-    * Whether the value being converted is a "remote" path or not. For instance if a task returns a value of
+    * Whether the value being converted is a "remote" path or not. For instance, if a task returns a value of
       "http://www.google.com" as a ``FlyteFile``, obviously it doesn't make sense for us to try to upload that to the
-      Flyte blob store. So no remote paths are uploaded. flytekit considers a path remote if it starts with ``s3://``,
+      Flyte blob store. So no remote paths are uploaded. Flytekit considers a path remote if it starts with ``s3://``,
       ``gs://``, ``http(s)://``, or even ``file://``.
 
     -----------
@@ -138,11 +148,15 @@ class FlyteFile(os.PathLike, typing.Generic[T]):
     def extension(cls) -> str:
         return ""
 
-    def __class_getitem__(cls, item: typing.Type) -> typing.Type[FlyteFile]:
+    def __class_getitem__(cls, item: typing.Union[str, typing.Type]) -> typing.Type[FlyteFile]:
+        from . import FileExt
+
         if item is None:
             return cls
-        item = str(item)
-        item = item.strip().lstrip("~").lstrip(".")
+
+        item_string = FileExt.check_and_convert_to_str(item)
+
+        item_string = item_string.strip().lstrip("~").lstrip(".")
         if item == "":
             return cls
 
@@ -152,18 +166,25 @@ class FlyteFile(os.PathLike, typing.Generic[T]):
 
             @classmethod
             def extension(cls) -> str:
-                return item
+                return item_string
 
         return _SpecificFormatClass
 
-    def __init__(self, path: str, downloader: typing.Callable = noop, remote_path=None):
+    def __init__(
+        self,
+        path: typing.Union[str, os.PathLike],
+        downloader: typing.Callable = noop,
+        remote_path: typing.Optional[os.PathLike] = None,
+    ):
         """
         :param path: The source path that users are expected to call open() on
         :param downloader: Optional function that can be passed that used to delay downloading of the actual fil
             until a user actually calls open().
         :param remote_path: If the user wants to return something and also specify where it should be uploaded to.
         """
-        self._path = path
+        # Make this field public, so that the dataclass transformer can set a value for it
+        # https://github.com/flyteorg/flytekit/blob/bcc8541bd6227b532f8462563fe8aac902242b21/flytekit/core/type_engine.py#L298
+        self.path = path
         self._downloader = downloader
         self._downloaded = False
         self._remote_path = remote_path
@@ -174,53 +195,42 @@ class FlyteFile(os.PathLike, typing.Generic[T]):
         if not self._downloaded:
             self._downloader()
             self._downloaded = True
-        return self._path
+        return self.path
 
     def __eq__(self, other):
         if isinstance(other, FlyteFile):
             return (
-                self._path == other._path
+                self.path == other.path
                 and self._remote_path == other._remote_path
                 and self.extension() == other.extension()
             )
         else:
-            return self._path == other
+            return self.path == other
 
     @property
     def downloaded(self) -> bool:
         return self._downloaded
 
     @property
-    def remote_path(self) -> typing.Optional[str]:
+    def remote_path(self) -> typing.Optional[os.PathLike]:
         return self._remote_path
 
-    @property
-    def path(self) -> str:
-        return self._path
-
     @property
     def remote_source(self) -> str:
         """
-        If this is an input to a task, and the original path is ``s3://something``, flytekit will download the
+        If this is an input to a task, and the original path is an ``s3`` bucket, Flytekit downloads the
         file for the user. In case the user wants access to the original path, it will be here.
         """
-        return self._remote_source
+        return typing.cast(str, self._remote_source)
 
     def download(self) -> str:
-        if self._downloaded:
-            return self.path
-        if self._downloader is not noop:
-            self._downloader()
-            self._downloaded = True
-            return self.path
-        else:
-            raise ValueError(f"Attempting to trigger download on non-downloadable file {self}")
+        return self.__fspath__()
 
     def __repr__(self):
-        return self._path
+        return self.path
 
     def __str__(self):
-        return self._path
+        return self.path
 
 
 class FlyteFilePathTransformer(TypeTransformer[FlyteFile]):
@@ -231,13 +241,23 @@ class FlyteFilePathTransformer(TypeTransformer[FlyteFile]):
     def get_format(t: typing.Union[typing.Type[FlyteFile], os.PathLike]) -> str:
         if t is os.PathLike:
             return ""
-        return t.extension()
+        return typing.cast(FlyteFile, t).extension()
 
-    def _blob_type(self, format: str) -> _core_types.BlobType:
-        return _core_types.BlobType(format=format, dimensionality=_core_types.BlobType.BlobDimensionality.SINGLE)
+    def _blob_type(self, format: str) -> BlobType:
+        return BlobType(format=format, dimensionality=BlobType.BlobDimensionality.SINGLE)
+
+    def assert_type(
+        self, t: typing.Union[typing.Type[FlyteFile], os.PathLike], v: typing.Union[FlyteFile, os.PathLike, str]
+    ):
+        if isinstance(v, os.PathLike) or isinstance(v, FlyteFile) or isinstance(v, str):
+            return
+        raise TypeError(
+            f"No automatic conversion found from type {type(v)} to FlyteFile."
+            f"Supported (os.PathLike, str, Flytefile)"
+        )
 
     def get_literal_type(self, t: typing.Union[typing.Type[FlyteFile], os.PathLike]) -> LiteralType:
-        return _type_models.LiteralType(blob=self._blob_type(format=FlyteFilePathTransformer.get_format(t)))
+        return LiteralType(blob=self._blob_type(format=FlyteFilePathTransformer.get_format(t)))
 
     def to_literal(
         self,
@@ -250,7 +270,7 @@ class FlyteFilePathTransformer(TypeTransformer[FlyteFile]):
         should_upload = True
 
         if python_val is None:
-            raise AssertionError("None value cannot be converted to a file.")
+            raise TypeTransformerFailedError("None value cannot be converted to a file.")
 
         if not (python_type is os.PathLike or issubclass(python_type, FlyteFile)):
             raise ValueError(f"Incorrect type {python_type}, must be either a FlyteFile or os.PathLike")
@@ -295,13 +315,13 @@ class FlyteFilePathTransformer(TypeTransformer[FlyteFile]):
                     if isinstance(python_val, str):
                         p = pathlib.Path(python_val)
                         if not p.is_file():
-                            raise ValueError(f"Error converting {python_val} because it's not a file.")
+                            raise TypeTransformerFailedError(f"Error converting {python_val} because it's not a file.")
             # python_type must be os.PathLike - see check at beginning of function
             else:
                 should_upload = False
 
         else:
-            raise AssertionError(f"Expected FlyteFile or os.PathLike object, received {type(python_val)}")
+            raise TypeTransformerFailedError(f"Expected FlyteFile or os.PathLike object, received {type(python_val)}")
 
         # If we're uploading something, that means that the uri should always point to the upload destination.
         if should_upload:
@@ -314,11 +334,12 @@ class FlyteFilePathTransformer(TypeTransformer[FlyteFile]):
             return Literal(scalar=Scalar(blob=Blob(metadata=meta, uri=source_path)))
 
     def to_python_value(
-        self, ctx: FlyteContext, lv: Literal, expected_python_type: typing.Union[typing.Type[FlyteFile]]
+        self, ctx: FlyteContext, lv: Literal, expected_python_type: typing.Union[typing.Type[FlyteFile], os.PathLike]
     ) -> FlyteFile:
-
-        uri = lv.scalar.blob.uri
-
+        try:
+            uri = lv.scalar.blob.uri
+        except AttributeError:
+            raise TypeTransformerFailedError(f"Cannot convert from {lv} to {expected_python_type}")
         # In this condition, we still return a FlyteFile instance, but it's a simple one that has no downloading tricks
         # Using is instead of issubclass because FlyteFile does actually subclass it
         if expected_python_type is os.PathLike:
@@ -340,17 +361,19 @@ class FlyteFilePathTransformer(TypeTransformer[FlyteFile]):
             return ctx.file_access.get_data(uri, local_path, is_multipart=False)
 
         expected_format = FlyteFilePathTransformer.get_format(expected_python_type)
-        ff = FlyteFile[expected_format](local_path, _downloader)
+        ff = FlyteFile.__class_getitem__(expected_format)(local_path, _downloader)
         ff._remote_source = uri
 
         return ff
 
-    def guess_python_type(self, literal_type: LiteralType) -> typing.Type[T]:
+    def guess_python_type(self, literal_type: LiteralType) -> typing.Type[FlyteFile[typing.Any]]:
         if (
             literal_type.blob is not None
-            and literal_type.blob.dimensionality == _core_types.BlobType.BlobDimensionality.SINGLE
+            and literal_type.blob.dimensionality == BlobType.BlobDimensionality.SINGLE
+            and literal_type.blob.format != FlytePickleTransformer.PYTHON_PICKLE_FORMAT
         ):
-            return FlyteFile[literal_type.blob.format]
+            return FlyteFile.__class_getitem__(literal_type.blob.format)
+
         raise ValueError(f"Transformer {self} cannot reverse {literal_type}")
 
 
