@@ -2,19 +2,19 @@ import os
 import typing
 from typing import Type
 
+import pandas
+
 from flytekit import FlyteContext
-from flytekit.configuration import sdk
 from flytekit.core.type_engine import T, TypeEngine, TypeTransformer
 from flytekit.models.literals import Literal, Scalar, Schema
 from flytekit.models.types import LiteralType, SchemaType
-from flytekit.plugins import pandas
 from flytekit.types.schema import LocalIOSchemaReader, LocalIOSchemaWriter, SchemaEngine, SchemaFormat, SchemaHandler
 
 
 class ParquetIO(object):
     PARQUET_ENGINE = "pyarrow"
 
-    def _read(self, chunk: os.PathLike, columns: typing.List[str], **kwargs) -> pandas.DataFrame:
+    def _read(self, chunk: os.PathLike, columns: typing.Optional[typing.List[str]], **kwargs) -> pandas.DataFrame:
         return pandas.read_parquet(chunk, columns=columns, engine=self.PARQUET_ENGINE, **kwargs)
 
     def read(self, *files: os.PathLike, columns: typing.List[str] = None, **kwargs) -> pandas.DataFrame:
@@ -55,40 +55,10 @@ class ParquetIO(object):
         )
 
 
-class FastParquetIO(ParquetIO):
-    PARQUET_ENGINE = "fastparquet"
-
-    def _read(self, chunk: os.PathLike, columns: typing.List[str], **kwargs) -> pandas.DataFrame:
-        from fastparquet import ParquetFile as _ParquetFile
-        from fastparquet import thrift_structures as _ts
-
-        # TODO Follow up to figure out if this is not needed anymore
-        # https://github.com/dask/fastparquet/issues/414#issuecomment-478983811
-        df = pandas.read_parquet(chunk, columns=columns, engine=self.PARQUET_ENGINE, index=False)
-        df_column_types = df.dtypes
-        pf = _ParquetFile(chunk)
-        schema_column_dtypes = {l.name: l.type for l in list(pf.schema.schema_elements)}
-
-        for idx in df_column_types[df_column_types == "float16"].index.tolist():
-            # A hacky way to get the string representations of the column types of a parquet schema
-            # Reference:
-            # https://github.com/dask/fastparquet/blob/f4ecc67f50e7bf98b2d0099c9589c615ea4b06aa/fastparquet/schema.py
-            if _ts.parquet_thrift.Type._VALUES_TO_NAMES[schema_column_dtypes[idx]] == "BOOLEAN":
-                df[idx] = df[idx].astype("object")
-                df[idx].replace({0: False, 1: True, pandas.np.nan: None}, inplace=True)
-        return df
-
-
-_PARQUETIO_ENGINES: typing.Dict[str, ParquetIO] = {
-    ParquetIO.PARQUET_ENGINE: ParquetIO(),
-    FastParquetIO.PARQUET_ENGINE: FastParquetIO(),
-}
-
-
 class PandasSchemaReader(LocalIOSchemaReader[pandas.DataFrame]):
     def __init__(self, local_dir: os.PathLike, cols: typing.Optional[typing.Dict[str, type]], fmt: SchemaFormat):
         super().__init__(local_dir, cols, fmt)
-        self._parquet_engine = _PARQUETIO_ENGINES[sdk.PARQUET_ENGINE.get()]
+        self._parquet_engine = ParquetIO()
 
     def _read(self, *path: os.PathLike, **kwargs) -> pandas.DataFrame:
         return self._parquet_engine.read(*path, columns=self.column_names, **kwargs)
@@ -97,7 +67,7 @@ class PandasSchemaReader(LocalIOSchemaReader[pandas.DataFrame]):
 class PandasSchemaWriter(LocalIOSchemaWriter[pandas.DataFrame]):
     def __init__(self, local_dir: os.PathLike, cols: typing.Optional[typing.Dict[str, type]], fmt: SchemaFormat):
         super().__init__(local_dir, cols, fmt)
-        self._parquet_engine = _PARQUETIO_ENGINES[sdk.PARQUET_ENGINE.get()]
+        self._parquet_engine = ParquetIO()
 
     def _write(self, df: T, path: os.PathLike, **kwargs):
         return self._parquet_engine.write(df, to_file=path, **kwargs)
@@ -110,7 +80,7 @@ class PandasDataFrameTransformer(TypeTransformer[pandas.DataFrame]):
 
     def __init__(self):
         super().__init__("PandasDataFrame<->GenericSchema", pandas.DataFrame)
-        self._parquet_engine = _PARQUETIO_ENGINES[sdk.PARQUET_ENGINE.get()]
+        self._parquet_engine = ParquetIO()
 
     @staticmethod
     def _get_schema_type() -> SchemaType:
@@ -143,6 +113,9 @@ class PandasDataFrameTransformer(TypeTransformer[pandas.DataFrame]):
         r = PandasSchemaReader(local_dir=local_dir, cols=None, fmt=SchemaFormat.PARQUET)
         return r.all()
 
+    def to_html(self, ctx: FlyteContext, python_val: pandas.DataFrame, expected_python_type: Type[T]):
+        return python_val.describe().to_html()
+
 
 SchemaEngine.register_handler(
     SchemaHandler("pandas-dataframe-schema", pandas.DataFrame, PandasSchemaReader, PandasSchemaWriter)
